@@ -35,7 +35,6 @@ import math
 import os
 import urllib2
 
-from ganeti import compat
 from ganeti import constants
 from ganeti import errors
 from ganeti import hypervisor
@@ -126,10 +125,11 @@ def CheckNodeGroupInstances(cfg, group_uuid, owned_instance_names):
   wanted_instances = frozenset(cfg.GetInstanceNames(
                                  cfg.GetNodeGroupInstances(group_uuid)))
   if owned_instance_names != wanted_instances:
+    group_name = cfg.GetNodeGroup(group_uuid).name
     raise errors.OpPrereqError("Instances in node group '%s' changed since"
                                " locks were acquired, wanted '%s', have '%s';"
                                " retry the operation" %
-                               (group_uuid,
+                               (group_name,
                                 utils.CommaJoin(wanted_instances),
                                 utils.CommaJoin(owned_instance_names)),
                                errors.ECODE_STATE)
@@ -511,7 +511,6 @@ def AdjustCandidatePool(
     lu.LogInfo("Promoted nodes to master candidate role: %s",
                utils.CommaJoin(node.name for node in mod_list))
     for node in mod_list:
-      lu.context.ReaddNode(node)
       AddNodeCertToCandidateCerts(lu, lu.cfg, node.uuid)
       if modify_ssh_setup:
         AddMasterCandidateSshKey(
@@ -999,9 +998,13 @@ def LoadNodeEvacResult(lu, alloc_result, early_release, use_nodes):
                  (name, _NodeEvacDest(use_nodes, group, node_names))
                  for (name, group, node_names) in moved))
 
-  return [map(compat.partial(_SetOpEarlyRelease, early_release),
-              map(opcodes.OpCode.LoadOpCode, ops))
-          for ops in jobs]
+  return [
+    [
+      _SetOpEarlyRelease(early_release, opcodes.OpCode.LoadOpCode(o))
+      for o in ops
+    ]
+    for ops in jobs
+  ]
 
 
 def _NodeEvacDest(use_nodes, group, node_names):
@@ -1533,7 +1536,7 @@ def DetermineImageSize(lu, image, node_uuid):
   return math.ceil(byte_size / 1024. / 1024.)
 
 
-def EnsureKvmdOnNodes(lu, feedback_fn, nodes=None):
+def EnsureKvmdOnNodes(lu, feedback_fn, nodes=None, silent_stop=False):
   """Ensure KVM daemon is running on nodes with KVM instances.
 
   If user shutdown is enabled in the cluster:
@@ -1556,6 +1559,9 @@ def EnsureKvmdOnNodes(lu, feedback_fn, nodes=None):
   @param nodes: if supplied, it overrides the node uuids to start/stop;
                 this is used mainly for optimization
 
+  @type silent_stop: bool
+  @param silent_stop: if we should suppress warnings in case KVM daemon is
+                      already stopped
   """
   cluster = lu.cfg.GetClusterInfo()
 
@@ -1584,15 +1590,16 @@ def EnsureKvmdOnNodes(lu, feedback_fn, nodes=None):
   if start_nodes:
     results = lu.rpc.call_node_ensure_daemon(start_nodes, constants.KVMD, True)
     for node_uuid in start_nodes:
-      results[node_uuid].Warn("Failed to start KVM daemon in node '%s'" %
-                              node_uuid, feedback_fn)
+      results[node_uuid].Warn("Failed to start KVM daemon on node '%s'" %
+                              lu.cfg.GetNodeName(node_uuid), feedback_fn)
 
   # Stop KVM where necessary
   if stop_nodes:
     results = lu.rpc.call_node_ensure_daemon(stop_nodes, constants.KVMD, False)
-    for node_uuid in stop_nodes:
-      results[node_uuid].Warn("Failed to stop KVM daemon in node '%s'" %
-                              node_uuid, feedback_fn)
+    if not silent_stop:
+      for node_uuid in stop_nodes:
+        results[node_uuid].Warn("Failed to stop KVM daemon on node '%s'" %
+                                lu.cfg.GetNodeName(node_uuid), feedback_fn)
 
 
 def WarnAboutFailedSshUpdates(result, master_uuid, feedback_fn):
