@@ -137,7 +137,7 @@ def GetInterfaceIpAddresses(ifname):
 
   """
   result = utils.RunCmd([constants.IP_COMMAND_PATH, "-o", "addr", "show",
-                         ifname])
+                         "dev", ifname])
 
   if result.failed:
     logging.error("Error running the ip command while getting the IP"
@@ -428,7 +428,7 @@ class IPAddress(object):
     @return: True if valid, False otherwise
 
     """
-    assert (isinstance(netmask, (int, long)))
+    assert isinstance(netmask, (int, long))
 
     return 0 < netmask <= cls.iplen
 
