@@ -78,14 +78,14 @@ class LogicalVolume(base.BlockDev):
   _INVALID_NAMES = compat.UniqueFrozenset([".", "..", "snapshot", "pvmove"])
   _INVALID_SUBSTRINGS = compat.UniqueFrozenset(["_mlog", "_mimage"])
 
-  def __init__(self, unique_id, children, size, params, dyn_params, *args):
+  def __init__(self, unique_id, children, size, params, dyn_params, **kwargs):
     """Attaches to a LV device.
 
     The unique_id is a tuple (vg_name, lv_name)
 
     """
     super(LogicalVolume, self).__init__(unique_id, children, size, params,
-                                        dyn_params, *args)
+                                        dyn_params, **kwargs)
     if not isinstance(unique_id, (tuple, list)) or len(unique_id) != 2:
       raise ValueError("Invalid configuration data %s" % str(unique_id))
     self._vg_name, self._lv_name = unique_id
@@ -95,7 +95,12 @@ class LogicalVolume(base.BlockDev):
     self._degraded = True
     self.major = self.minor = self.pe_size = self.stripe_count = None
     self.pv_names = None
-    self.Attach()
+    lvs_cache = kwargs.get("lvs_cache")
+    if lvs_cache:
+      lv_info = lvs_cache.get(self.dev_path)
+      self.Attach(lv_info=lv_info)
+    else:
+      self.Attach()
 
   @staticmethod
   def _GetStdPvSize(pvs_info):
@@ -132,11 +137,11 @@ class LogicalVolume(base.BlockDev):
     empty_pvs = filter(objects.LvmPvInfo.IsEmpty, pvs_info)
     if max_pvs is not None:
       empty_pvs = empty_pvs[:max_pvs]
-    return map((lambda pv: pv.name), empty_pvs)
+    return [pv.name for pv in empty_pvs]
 
   @classmethod
   def Create(cls, unique_id, children, size, spindles, params, excl_stor,
-             dyn_params, *args):
+             dyn_params, **kwargs):
     """Create a new logical volume.
 
     """
@@ -218,7 +223,8 @@ class LogicalVolume(base.BlockDev):
     if result.failed:
       base.ThrowError("LV create failed (%s): %s",
                       result.fail_reason, result.output)
-    return LogicalVolume(unique_id, children, size, params, dyn_params, *args)
+    return LogicalVolume(unique_id, children, size, params,
+                         dyn_params, **kwargs)
 
   @staticmethod
   def _GetVolumeInfo(lvm_cmd, fields):
@@ -436,9 +442,9 @@ class LogicalVolume(base.BlockDev):
     self._lv_name = new_name
     self.dev_path = utils.PathJoin("/dev", self._vg_name, self._lv_name)
 
-  @classmethod
-  def _ParseLvInfoLine(cls, line, sep):
-    """Parse one line of the lvs output used in L{_GetLvInfo}.
+  @staticmethod
+  def _ParseLvInfoLine(line, sep):
+    """Parse one line of the lvs output used in L{GetLvGlobalInfo}.
 
     """
     elems = line.strip().split(sep)
@@ -447,13 +453,14 @@ class LogicalVolume(base.BlockDev):
     # separator to the right of the output. The PV info might be empty for
     # thin volumes, so stripping off the separators might cut off the last
     # empty element - do this instead.
-    if len(elems) == 7 and elems[-1] == "":
+    if len(elems) == 9 and elems[-1] == "":
       elems.pop()
 
-    if len(elems) != 6:
-      base.ThrowError("Can't parse LVS output, len(%s) != 6", str(elems))
+    if len(elems) != 8:
+      base.ThrowError("Can't parse LVS output, len(%s) != 8", str(elems))
 
-    (status, major, minor, pe_size, stripes, pvs) = elems
+    (vg_name, lv_name, status, major, minor, pe_size, stripes, pvs) = elems
+    path = os.path.join(os.environ.get('DM_DEV_DIR', '/dev'), vg_name, lv_name)
     if len(status) < 6:
       base.ThrowError("lvs lv_attr is not at least 6 characters (%s)", status)
 
@@ -476,43 +483,37 @@ class LogicalVolume(base.BlockDev):
     pv_names = []
     if pvs != "":
       for pv in pvs.split(","):
-        m = re.match(cls._PARSE_PV_DEV_RE, pv)
+        m = re.match(LogicalVolume._PARSE_PV_DEV_RE, pv)
         if not m:
           base.ThrowError("Can't parse this device list: %s", pvs)
         pv_names.append(m.group(1))
 
-    return (status, major, minor, pe_size, stripes, pv_names)
+    return (path, (status, major, minor, pe_size, stripes, pv_names))
 
-  @classmethod
-  def _GetLvInfo(cls, dev_path, _run_cmd=utils.RunCmd):
-    """Get info about the given existing LV to be used.
+  @staticmethod
+  def GetLvGlobalInfo(_run_cmd=utils.RunCmd):
+    """Obtain the current state of the existing LV disks.
+
+    @return: a dict containing the state of each disk with the disk path as key
 
     """
     sep = "|"
     result = _run_cmd(["lvs", "--noheadings", "--separator=%s" % sep,
                        "--units=k", "--nosuffix",
-                       "-olv_attr,lv_kernel_major,lv_kernel_minor,"
-                       "vg_extent_size,stripes,devices", dev_path])
+                       "-ovg_name,lv_name,lv_attr,lv_kernel_major,"
+                       "lv_kernel_minor,vg_extent_size,stripes,devices"])
     if result.failed:
-      base.ThrowError("Can't find LV %s: %s, %s",
-                      dev_path, result.fail_reason, result.output)
-    # the output can (and will) have multiple lines for multi-segment
-    # LVs, as the 'stripes' parameter is a segment one, so we take
-    # only the last entry, which is the one we're interested in; note
-    # that with LVM2 anyway the 'stripes' value must be constant
-    # across segments, so this is a no-op actually
+      logging.warning("lvs command failed, the LV cache will be empty!")
+      logging.info("lvs failure: %r", result.stderr)
+      return {}
     out = result.stdout.splitlines()
-    if not out: # totally empty result? splitlines() returns at least
-                # one line for any non-empty string
-      base.ThrowError("Can't parse LVS output, no lines? Got '%s'", str(out))
-    pv_names = set()
-    for line in out:
-      (status, major, minor, pe_size, stripes, more_pvs) = \
-        cls._ParseLvInfoLine(line, sep)
-      pv_names.update(more_pvs)
-    return (status, major, minor, pe_size, stripes, pv_names)
+    if not out:
+      logging.warning("lvs command returned an empty output, the LV cache will"
+                      "be empty!")
+      return {}
+    return dict([LogicalVolume._ParseLvInfoLine(line, sep) for line in out])
 
-  def Attach(self):
+  def Attach(self, lv_info=None, **kwargs):
     """Attach to an existing LV.
 
     This method will try to see if an existing and active LV exists
@@ -521,11 +522,11 @@ class LogicalVolume(base.BlockDev):
 
     """
     self.attached = False
-    try:
-      (status, major, minor, pe_size, stripes, pv_names) = \
-        self._GetLvInfo(self.dev_path)
-    except errors.BlockDeviceError:
+    if not lv_info:
+      lv_info = LogicalVolume.GetLvGlobalInfo().get(self.dev_path)
+    if not lv_info:
       return False
+    (status, major, minor, pe_size, stripes, pv_names) = lv_info
 
     self.major = major
     self.minor = minor
@@ -747,14 +748,14 @@ class PersistentBlockDevice(base.BlockDev):
   For the time being, pathnames are required to lie under /dev.
 
   """
-  def __init__(self, unique_id, children, size, params, dyn_params, *args):
+  def __init__(self, unique_id, children, size, params, dyn_params, **kwargs):
     """Attaches to a static block device.
 
     The unique_id is a path under /dev.
 
     """
     super(PersistentBlockDevice, self).__init__(unique_id, children, size,
-                                                params, dyn_params, *args)
+                                                params, dyn_params, **kwargs)
     if not isinstance(unique_id, (tuple, list)) or len(unique_id) != 2:
       raise ValueError("Invalid configuration data %s" % str(unique_id))
     self.dev_path = unique_id[1]
@@ -774,7 +775,7 @@ class PersistentBlockDevice(base.BlockDev):
 
   @classmethod
   def Create(cls, unique_id, children, size, spindles, params, excl_stor,
-             dyn_params, *args):
+             dyn_params, **kwargs):
     """Create a new device
 
     This is a noop, we only return a PersistentBlockDevice instance
@@ -784,7 +785,7 @@ class PersistentBlockDevice(base.BlockDev):
       raise errors.ProgrammerError("Persistent block device requested with"
                                    " exclusive_storage")
     return PersistentBlockDevice(unique_id, children, 0, params, dyn_params,
-                                 *args)
+                                 **kwargs)
 
   def Remove(self):
     """Remove a device
@@ -803,7 +804,6 @@ class PersistentBlockDevice(base.BlockDev):
   def Attach(self):
     """Attach to an existing block device.
 
-
     """
     self.attached = False
     try:
@@ -870,12 +870,12 @@ class RADOSBlockDevice(base.BlockDev):
   this to be functional.
 
   """
-  def __init__(self, unique_id, children, size, params, dyn_params, *args):
+  def __init__(self, unique_id, children, size, params, dyn_params, **kwargs):
     """Attaches to an rbd device.
 
     """
     super(RADOSBlockDevice, self).__init__(unique_id, children, size, params,
-                                           dyn_params, *args)
+                                           dyn_params, **kwargs)
     if not isinstance(unique_id, (tuple, list)) or len(unique_id) != 2:
       raise ValueError("Invalid configuration data %s" % str(unique_id))
 
@@ -887,7 +887,7 @@ class RADOSBlockDevice(base.BlockDev):
 
   @classmethod
   def Create(cls, unique_id, children, size, spindles, params, excl_stor,
-             dyn_params, *args):
+             dyn_params, **kwargs):
     """Create a new rbd device.
 
     Provision a new rbd volume inside a RADOS pool.
@@ -911,7 +911,7 @@ class RADOSBlockDevice(base.BlockDev):
                       result.fail_reason, result.output)
 
     return RADOSBlockDevice(unique_id, children, size, params, dyn_params,
-                            *args)
+                            **kwargs)
 
   def Remove(self):
     """Remove the rbd device.
@@ -1020,8 +1020,6 @@ class RADOSBlockDevice(base.BlockDev):
       showmap_cmd = [
         constants.RBD_CMD,
         "showmapped",
-        "-p",
-        pool,
         "--format",
         "json"
         ]
@@ -1032,7 +1030,7 @@ class RADOSBlockDevice(base.BlockDev):
                       result.fail_reason, result.output)
         raise RbdShowmappedJsonError
 
-      return cls._ParseRbdShowmappedJson(result.output, volume_name)
+      return cls._ParseRbdShowmappedJson(result.output, pool, volume_name)
     except RbdShowmappedJsonError:
       # For older versions of rbd, we have to parse the plain / text output
       # manually.
@@ -1045,7 +1043,7 @@ class RADOSBlockDevice(base.BlockDev):
       return cls._ParseRbdShowmappedPlain(result.output, volume_name)
 
   @staticmethod
-  def _ParseRbdShowmappedJson(output, volume_name):
+  def _ParseRbdShowmappedJson(output, volume_pool, volume_name):
     """Parse the json output of `rbd showmapped'.
 
     This method parses the json output of `rbd showmapped' and returns the rbd
@@ -1053,8 +1051,10 @@ class RADOSBlockDevice(base.BlockDev):
 
     @type output: string
     @param output: the json output of `rbd showmapped'
+    @type volume_pool: string
+    @param volume_pool: name of the volume whose device we search for
     @type volume_name: string
-    @param volume_name: the name of the volume whose device we search for
+    @param volume_name: name of the pool in which we search
     @rtype: string or None
     @return: block device path if the volume is mapped, else None
 
@@ -1064,14 +1064,23 @@ class RADOSBlockDevice(base.BlockDev):
     except ValueError, err:
       base.ThrowError("Unable to parse JSON data: %s" % err)
 
+    # since ceph mimic the json output changed from dict to list
+    if isinstance(devices, dict):
+      devices = devices.values()
+
     rbd_dev = None
-    for d in devices.values(): # pylint: disable=E1103
+    for d in devices:
       try:
         name = d["name"]
       except KeyError:
         base.ThrowError("'name' key missing from json object %s", devices)
 
-      if name == volume_name:
+      try:
+        pool = d["pool"]
+      except KeyError:
+        base.ThrowError("'pool' key missing from json object %s", devices)
+
+      if name == volume_name and pool == volume_pool:
         if rbd_dev is not None:
           base.ThrowError("rbd volume %s is mapped more than once", volume_name)
 
@@ -1102,7 +1111,7 @@ class RADOSBlockDevice(base.BlockDev):
     lines = output.splitlines()
 
     # Try parsing the new output format (ceph >= 0.55).
-    splitted_lines = map(lambda l: l.split(), lines)
+    splitted_lines = [l.split() for l in lines]
 
     # Check for empty output.
     if not splitted_lines:
@@ -1113,14 +1122,13 @@ class RADOSBlockDevice(base.BlockDev):
     if field_cnt != allfields:
       # Parsing the new format failed. Fallback to parsing the old output
       # format (< 0.55).
-      splitted_lines = map(lambda l: l.split("\t"), lines)
+      splitted_lines = [l.split("\t") for l in lines]
       if field_cnt != allfields:
         base.ThrowError("Cannot parse rbd showmapped output expected %s fields,"
                         " found %s", allfields, field_cnt)
 
-    matched_lines = \
-      filter(lambda l: len(l) == allfields and l[volumefield] == volume_name,
-             splitted_lines)
+    matched_lines = [l for l in splitted_lines
+                     if len(l) == allfields and l[volumefield] == volume_name]
 
     if len(matched_lines) > 1:
       base.ThrowError("rbd volume %s mapped more than once", volume_name)
@@ -1305,7 +1313,7 @@ def _VerifyDiskParams(disk):
                                  missing)
 
 
-def FindDevice(disk, children):
+def FindDevice(disk, children, **kwargs):
   """Search for an existing, assembled device.
 
   This will succeed only if the device exists and is assembled, but it
@@ -1321,7 +1329,7 @@ def FindDevice(disk, children):
   _VerifyDiskType(disk.dev_type)
   device = DEV_MAP[disk.dev_type](disk.logical_id, children, disk.size,
                                   disk.params, disk.dynamic_params,
-                                  disk.name, disk.uuid)
+                                  name=disk.name, uuid=disk.uuid, **kwargs)
   if not device.attached:
     return None
   return device
@@ -1344,7 +1352,7 @@ def Assemble(disk, children):
   _VerifyDiskParams(disk)
   device = DEV_MAP[disk.dev_type](disk.logical_id, children, disk.size,
                                   disk.params, disk.dynamic_params,
-                                  disk.name, disk.uuid)
+                                  name=disk.name, uuid=disk.uuid)
   device.Assemble()
   return device
 
@@ -1368,7 +1376,7 @@ def Create(disk, children, excl_stor):
   device = DEV_MAP[disk.dev_type].Create(disk.logical_id, children, disk.size,
                                          disk.spindles, disk.params, excl_stor,
                                          disk.dynamic_params,
-                                         disk.name, disk.uuid)
+                                         name=disk.name, uuid=disk.uuid)
   return device
 
 # Please keep this at the bottom of the file for visibility.
