@@ -2,6 +2,7 @@
 {-# LANGUAGE FlexibleContexts #-}
 {-# LANGUAGE MultiParamTypeClasses #-}
 {-# LANGUAGE TypeFamilies #-}
+{-# LANGUAGE DeriveFunctor #-}
 {-# LANGUAGE UndecidableInstances #-}
 {-# LANGUAGE CPP #-}
 
@@ -55,6 +56,7 @@ module Ganeti.BasicTypes
   , justBad
   , eitherToResult
   , isLeft
+  , isRight
   , annotateResult
   , annotateError
   , failError
@@ -75,14 +77,6 @@ module Ganeti.BasicTypes
   , emptyListSet
   ) where
 
--- The following macro is just a temporary solution for 2.12 and 2.13.
--- Since 2.14 cabal creates proper macros for all dependencies.
-#define MIN_VERSION_monad_control(maj,min,rev) \
-  (((maj)<MONAD_CONTROL_MAJOR)|| \
-   (((maj)==MONAD_CONTROL_MAJOR)&&((min)<=MONAD_CONTROL_MINOR))|| \
-   (((maj)==MONAD_CONTROL_MAJOR)&&((min)==MONAD_CONTROL_MINOR)&& \
-    ((rev)<=MONAD_CONTROL_REV)))
-
 import Control.Applicative
 import Control.Exception (try)
 import Control.Monad
@@ -99,6 +93,10 @@ import qualified Data.Set as Set (empty)
 import Text.JSON (JSON)
 import qualified Text.JSON as JSON (readJSON, showJSON)
 
+-- Remove after we require >= 1.8.58
+-- See: https://github.com/ndmitchell/hlint/issues/24
+{-# ANN module "HLint: ignore Unused LANGUAGE pragma" #-}
+
 -- | Generic monad for our error handling mechanisms.
 data GenericResult a b
   = Bad a
@@ -125,13 +123,17 @@ instance Functor (GenericResult a) where
   fmap _ (Bad msg) = Bad msg
   fmap fn (Ok val) = Ok (fn val)
 
-instance (Error a, Monoid a) => MonadPlus (GenericResult a) where
-  mzero = Bad $ strMsg "zero Result when used as MonadPlus"
+instance (Error a, Monoid a) => Alternative (GenericResult a) where
+  empty = Bad $ strMsg "zero Result when used as empty"
   -- for mplus, when we 'add' two Bad values, we concatenate their
   -- error descriptions
-  (Bad x) `mplus` (Bad y) = Bad (x `mappend` strMsg "; " `mappend` y)
-  (Bad _) `mplus` x = x
-  x@(Ok _) `mplus` _ = x
+  (Bad x) <|> (Bad y) = Bad (x `mappend` strMsg "; " `mappend` y)
+  (Bad _) <|> x = x
+  x@(Ok _) <|> _ = x
+
+instance (Error a, Monoid a) => MonadPlus (GenericResult a) where
+  mzero = empty
+  mplus = (<|>)
 
 instance (Error a) => MonadError a (GenericResult a) where
   throwError = Bad
@@ -145,10 +147,6 @@ instance Applicative (GenericResult a) where
   _       <*> (Bad x) = Bad x
   (Ok f)  <*> (Ok x)  = Ok $ f x
 
-instance (Error a, Monoid a) => Alternative (GenericResult a) where
-  empty = mzero
-  (<|>) = mplus
-
 -- | This is a monad transformation for Result. It's implementation is
 -- based on the implementations of MaybeT and ErrorT.
 --
@@ -156,6 +154,7 @@ instance (Error a, Monoid a) => Alternative (GenericResult a) where
 -- If 'mplus' combines two failing operations, errors of both of them
 -- are combined.
 newtype ResultT a m b = ResultT {runResultT :: m (GenericResult a b)}
+  deriving (Functor)
 
 -- | Eliminates a 'ResultT' value given appropriate continuations
 elimResultT :: (Monad m)
@@ -169,10 +168,7 @@ elimResultT l r = ResultT . (runResultT . result <=< runResultT)
     result (Bad e)  = l e
 {-# INLINE elimResultT #-}
 
-instance (Monad f) => Functor (ResultT a f) where
-  fmap f = ResultT . liftM (fmap f) . runResultT
-
-instance (Monad m, Error a) => Applicative (ResultT a m) where
+instance (Applicative m, Monad m, Error a) => Applicative (ResultT a m) where
   pure = return
   (<*>) = ap
 
@@ -237,16 +233,18 @@ instance (Error a, MonadBaseControl IO m)
   {-# INLINE liftBaseWith #-}
   {-# INLINE restoreM #-}
 
-instance (Monad m, Error a, Monoid a) => MonadPlus (ResultT a m) where
-  mzero = ResultT $ return mzero
+instance (Monad m, Applicative m, Error a, Monoid a)
+         => Alternative (ResultT a m) where
+  empty = ResultT $ return mzero
   -- Ensure that 'y' isn't run if 'x' contains a value. This makes it a bit
   -- more complicated than 'mplus' of 'GenericResult'.
-  mplus x y = elimResultT combine return x
+  x <|> y = elimResultT combine return x
     where combine x' = ResultT $ liftM (mplus (Bad x')) (runResultT y)
 
-instance (Monad m, Error a, Monoid a) => Alternative (ResultT a m) where
-  empty = mzero
-  (<|>) = mplus
+instance (Monad m, Applicative m, Error a, Monoid a)
+         => MonadPlus (ResultT a m) where
+  mzero = empty
+  mplus = (<|>)
 
 -- | Changes the error message of a result value, if present.
 -- Note that since 'GenericResult' is also a 'MonadError', this function
@@ -323,6 +321,11 @@ isLeft :: Either a b -> Bool
 isLeft (Left _) = True
 isLeft _        = False
 
+-- | Check if an either is Right. Equivalent to isRight from Data.Either
+-- version 4.7.0.0 or higher.
+isRight :: Either a b -> Bool
+isRight = not . isLeft
+
 -- | Annotate an error with an ownership information, lifting it to a
 -- 'MonadError'. Since 'Result' is an instance of 'MonadError' itself,
 -- it's a generalization of type @String -> Result a -> Result a@.
@@ -461,7 +464,7 @@ lookupName l s = foldr (chooseLookupResult s)
 -- opcode generation to transform a Haskell 'Set' into a Python 'list'
 -- without duplicate elements.
 newtype ListSet a = ListSet { unListSet :: Set a }
-  deriving (Eq, Show)
+  deriving (Eq, Show, Ord)
 
 instance (Ord a, JSON a) => JSON (ListSet a) where
   showJSON = JSON.showJSON . unListSet
