@@ -36,7 +36,6 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 module Ganeti.JQScheduler
   ( JQStatus
   , jqLivelock
-  , jqForkLock
   , emptyJQStatus
   , selectJobsToRun
   , scheduleSomeJobs
@@ -45,6 +44,7 @@ module Ganeti.JQScheduler
   , dequeueJob
   , setJobPriority
   , cleanupIfDead
+  , updateStatusAndScheduleSomeJobs
   , configChangeNeedsRescheduling
   ) where
 
@@ -56,7 +56,7 @@ import Control.Monad
 import Control.Monad.IO.Class
 import Data.Function (on)
 import Data.Functor ((<$))
-import Data.IORef
+import Data.IORef (IORef, atomicModifyIORef, newIORef, readIORef)
 import Data.List
 import Data.Maybe
 import qualified Data.Map as Map
@@ -66,6 +66,7 @@ import qualified Data.Set as S
 import System.INotify
 
 import Ganeti.BasicTypes
+import Ganeti.Compat
 import Ganeti.Constants as C
 import Ganeti.Errors
 import Ganeti.JQScheduler.Filtering (applyingFilter, jobFiltering)
@@ -109,13 +110,23 @@ emptyJQStatus config = do
   return JQStatus { jqJobs = jqJ, jqConfig = config, jqLivelock = livelock
                   , jqForkLock = forkLock }
 
+-- When updating the job lists, force the elements to WHNF, otherwise it is
+-- easy to leak the resources held onto by the lazily parsed job file.
+-- This can happen, eg, if updateJob is called, but the resulting QueuedJob
+-- isn't used by the scheduler, for example when the inotify watcher or the
+-- the polling loop re-reads a job with a new message appended to it.
+
 -- | Apply a function on the running jobs.
 onRunningJobs :: ([JobWithStat] -> [JobWithStat]) -> Queue -> Queue
-onRunningJobs = over qRunningL
+onRunningJobs f q@Queue { qRunning = qr } =
+  let qr' = (foldr seq () qr) `seq` f qr -- force list els to WHNF
+  in q { qRunning = qr' }
 
 -- | Apply a function on the queued jobs.
 onQueuedJobs :: ([JobWithStat] -> [JobWithStat]) -> Queue -> Queue
-onQueuedJobs = over qEnqueuedL
+onQueuedJobs f q@Queue { qEnqueued = qe } =
+  let qe' = (foldr seq () qe) `seq` f qe -- force list els to WHNF
+  in q { qEnqueued = qe' }
 
 -- | Obtain a JobWithStat from a QueuedJob.
 unreadJob :: QueuedJob -> JobWithStat
@@ -150,7 +161,7 @@ getRQL = liftM (length . qRunning) . readIORef . jqJobs
 
 -- | Wrapper function to atomically update the jobs in the queue status.
 modifyJobs :: JQStatus -> (Queue -> Queue) -> IO ()
-modifyJobs qstat f = atomicModifyIORef (jqJobs qstat) (flip (,) ()  . f)
+modifyJobs qstat f = atomicModifyIORef' (jqJobs qstat) (flip (,) ()  . f)
 
 -- | Reread a job from disk, if the file has changed.
 readJobStatus :: JobWithStat -> IO (Maybe JobWithStat)
@@ -269,7 +280,7 @@ jobWatcher state jWS e = do
   let inotify = jINotify jWS
   when (e == Ignored  && isJust inotify) $ do
     qdir <- queueDir
-    let fpath = liveJobFile qdir jid
+    let fpath = toInotifyPath $ liveJobFile qdir jid
     _ <- addWatch (fromJust inotify) [Modify, Delete] fpath
            (jobWatcher state jWS)
     return ()
@@ -287,7 +298,8 @@ attachWatcher state jWS = when (isNothing $ jINotify jWS) $ do
      let fpath = liveJobFile qdir . qjId $ jJob jWS
          jWS' = jWS { jINotify=Just inotify }
      logDebug $ "Attaching queue watcher for " ++ fpath
-     _ <- addWatch inotify [Modify, Delete] fpath $ jobWatcher state jWS'
+     _ <- addWatch inotify [Modify, Delete] (toInotifyPath fpath)
+            $ jobWatcher state jWS'
      modifyJobs state . onRunningJobs $ updateJobStatus jWS'
    else logDebug $ "Not attaching watcher for job "
                    ++ (show . fromJobId . qjId $ jJob jWS)
@@ -473,11 +485,9 @@ cleanupIfDead state jid = do
   let jobWS = find ((==) jid . qjId . jJob) $ qRunning jobs
   maybe (return True) (checkForDeath state) jobWS
 
--- | Time-based watcher for updating the job queue.
-onTimeWatcher :: JQStatus -> IO ()
-onTimeWatcher qstate = forever $ do
-  threadDelay watchInterval
-  logDebug "Job queue watcher timer fired"
+-- | Force the queue to check the state of all jobs.
+updateStatusAndScheduleSomeJobs :: JQStatus -> IO ()
+updateStatusAndScheduleSomeJobs qstate =  do
   jobs <- readIORef (jqJobs qstate)
   mapM_ (checkForDeath qstate) $ qRunning jobs
   jobs' <- readIORef (jqJobs qstate)
@@ -486,6 +496,13 @@ onTimeWatcher qstate = forever $ do
   jobs'' <- readIORef (jqJobs qstate)
   logInfo $ showQueue jobs''
   scheduleSomeJobs qstate
+
+-- | Time-based watcher for updating the job queue.
+onTimeWatcher :: JQStatus -> IO ()
+onTimeWatcher qstate = forever $ do
+  threadDelay watchInterval
+  logDebug "Job queue watcher timer fired"
+  updateStatusAndScheduleSomeJobs qstate
   logDebug "Job queue watcher cycle finished"
 
 -- | Read a single, non-archived, job, specified by its id, from disk.
@@ -589,7 +606,8 @@ setJobPriority state jid prio = runResultT $ do
 -- | Given old and new configs, determines if the changes between them should
 -- trigger the scheduler to run.
 configChangeNeedsRescheduling :: ConfigData -> ConfigData -> Bool
-configChangeNeedsRescheduling oldConfig newConfig =
-  or -- Trigger rescheduling if:
-    [ configFilters oldConfig /= configFilters newConfig -- filters changed
-    ]
+configChangeNeedsRescheduling old new =
+  -- Trigger rescheduling if any of the following change:
+  (((/=) `on` configFilters) old new || -- filters
+   ((/=) `on` clusterMaxRunningJobs . configCluster) old new -- run queue length
+  )
