@@ -113,7 +113,7 @@ import qualified Ganeti.Constants as C
 import Ganeti.Errors (ErrorResult, ResultG)
 import Ganeti.JQueue.Lens (qoInputL, validOpCodeL)
 import Ganeti.JQueue.Objects
-import Ganeti.JSON
+import Ganeti.JSON (fromJResult, fromObjWithDefault)
 import Ganeti.Logging
 import Ganeti.Luxi
 import Ganeti.Objects (ConfigData, Node)
@@ -549,7 +549,7 @@ startJobs luxiLivelock forkLock jobs = do
         void . mkResultT . writeJobToDisk qdir
           $ job { qjLivelock = Just llfile }
   let runJob job = withLock forkLock $ do
-        (llfile, _) <- Exec.forkJobProcess (qjId job) luxiLivelock
+        (llfile, _) <- Exec.forkJobProcess job luxiLivelock
                                            (updateJob job)
         return $ job { qjLivelock = Just llfile }
   mapM (runResultT . runJob) jobs
@@ -563,9 +563,10 @@ isQueuedJobDead ownlivelock =
   . mfilter (/= ownlivelock)
   . qjLivelock
 
--- | Waits for a job to finalize its execution.
-waitForJob :: JobId -> Int -> ResultG (Bool, String)
-waitForJob jid tmout = do
+-- | Waits for a job ordered to cancel to react, and returns whether it was
+-- canceled, and a user-intended description of the reason.
+waitForJobCancelation :: JobId -> Int -> ResultG (Bool, String)
+waitForJobCancelation jid tmout = do
   qDir <- liftIO queueDir
   let jobfile = liveJobFile qDir jid
       load = liftM fst <$> loadJobFromDisk qDir False jid
@@ -610,7 +611,7 @@ cancelJob kill luxiLivelock jid = runResultT $ do
           if calcJobStatus job > JOB_STATUS_WAITING
             then return (False, "Job no longer waiting, can't cancel\
                                 \ (informed it anyway)")
-            else lift $ waitForJob jid C.luxiCancelJobTimeout
+            else lift $ waitForJobCancelation jid C.luxiCancelJobTimeout
           else return (True, "SIGKILL send to the process")
       _ -> do
         logDebug $ jName ++ " in its startup phase, retrying"
