@@ -95,6 +95,7 @@ import qualified Data.Traversable as F
 import Data.Maybe (fromMaybe, catMaybes)
 import qualified Data.Map as Map
 import qualified Data.Set as Set
+import qualified Data.Semigroup as Sem
 import System.Time (ClockTime(..))
 import Text.Printf (printf)
 
@@ -394,13 +395,16 @@ instance (HasStringRepr a, Ord a, J.JSON b) =>
 
 -- * Types that (de)serialize in a special form of JSON
 
-newtype UsedKeys = UsedKeys (Maybe (Set.Set String))
+newtype UsedKeys = UsedKeys (Maybe (Set.Set T.Text))
+
+instance Sem.Semigroup UsedKeys where
+  (UsedKeys xs) <> (UsedKeys ys) = UsedKeys $ liftA2 Set.union xs ys
 
 instance Monoid UsedKeys where
   mempty = UsedKeys (Just Set.empty)
-  mappend (UsedKeys xs) (UsedKeys ys) = UsedKeys $ liftA2 Set.union xs ys
+  mappend = (Sem.<>)
 
-mkUsedKeys :: Set.Set String -> UsedKeys
+mkUsedKeys :: Set.Set T.Text -> UsedKeys
 mkUsedKeys = UsedKeys . Just
 
 allUsedKeys :: UsedKeys
@@ -426,6 +430,7 @@ readJSONfromDict :: (DictObject a)
                  => J.JSValue -> J.Result a
 readJSONfromDict jsv = do
   dict <- liftM J.fromJSObject $ J.readJSON jsv
+
   (r, UsedKeys keys) <- runWriterT $ fromDictWKeys dict
   -- check that no superfluous dictionary keys are present
   case keys of
@@ -434,7 +439,7 @@ readJSONfromDict jsv = do
                ++ show (Set.toAscList superfluous) ++ ", but only "
                ++ show (Set.toAscList allowedSet) ++ " allowed."
       where
-        superfluous = Set.fromList (map fst dict) Set.\\ allowedSet
+        superfluous = Set.fromList (map (T.pack . fst) dict) Set.\\ allowedSet
     _ -> return ()
   return r
 
@@ -543,4 +548,4 @@ addField _ jsval = jsval
 
 -- | Maybe obtain a map from a JSON object.
 maybeParseMap :: J.JSON a => J.JSValue -> Maybe (Map.Map String a)
-maybeParseMap = liftM fromContainer . readContainer <=< asJSObject
+maybeParseMap =liftM fromContainer . readContainer <=< asJSObject
