@@ -30,21 +30,30 @@ already exist in the KVM control directory when the KVM daemon starts.
 {-
 
 Copyright (C) 2013 Google Inc.
+All rights reserved.
 
-This program is free software; you can redistribute it and/or modify
-it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+Redistribution and use in source and binary forms, with or without
+modification, are permitted provided that the following conditions are
+met:
 
-This program is distributed in the hope that it will be useful, but
-WITHOUT ANY WARRANTY; without even the implied warranty of
-MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-General Public License for more details.
+1. Redistributions of source code must retain the above copyright notice,
+this list of conditions and the following disclaimer.
 
-You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
-Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
-02110-1301, USA.
+2. Redistributions in binary form must reproduce the above copyright
+notice, this list of conditions and the following disclaimer in the
+documentation and/or other materials provided with the distribution.
+
+THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
+IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
+TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
+PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR
+CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 -}
 
@@ -67,6 +76,7 @@ import System.INotify
 
 import qualified AutoConf
 import qualified Ganeti.BasicTypes as BasicTypes
+import Ganeti.Compat
 import qualified Ganeti.Constants as Constants
 import qualified Ganeti.Daemon as Daemon (getFQDN)
 import qualified Ganeti.Logging as Logging
@@ -213,7 +223,7 @@ ensureMonitor monitors monitorFile =
 handleGenericEvent :: Lock -> String -> String -> Event -> IO ()
 handleGenericEvent lock curDir tarDir ev@Created {}
   | isDirectory ev && curDir /= tarDir &&
-    (curDir </> filePath ev) `isPrefixPath` tarDir = putMVar lock ()
+    (curDir </> filePath' ev) `isPrefixPath` tarDir = putMVar lock ()
 handleGenericEvent lock _ _ event
   | event == DeletedSelf || event == Unmounted = putMVar lock ()
 handleGenericEvent _ _ _ _ = return ()
@@ -224,23 +234,23 @@ handleGenericEvent _ _ _ _ = return ()
 -- ensures that there is a monitor running for the new Qmp socket.
 handleTargetEvent :: Lock -> Monitors -> String -> Event -> IO ()
 handleTargetEvent _ monitors tarDir ev@Created {}
-  | not (isDirectory ev) && isMonitorPath (filePath ev) =
-    ensureMonitor monitors $ tarDir </> filePath ev
+  | not (isDirectory ev) && isMonitorPath (filePath' ev) =
+    ensureMonitor monitors $ tarDir </> filePath' ev
 handleTargetEvent lock monitors tarDir ev@Opened {}
   | not (isDirectory ev) =
-    case maybeFilePath ev of
+    case maybeFilePath' ev of
       Just p | isMonitorPath p ->
-        ensureMonitor monitors $ tarDir </> filePath ev
+        ensureMonitor monitors $ tarDir </> filePath' ev
       _ ->
         handleGenericEvent lock tarDir tarDir ev
 handleTargetEvent _ _ tarDir ev@Created {}
-  | not (isDirectory ev) && takeExtension (filePath ev) == shutdownExtension =
+  | not (isDirectory ev) && takeExtension (filePath' ev) == shutdownExtension =
     Logging.logInfo $ "User shutdown file opened " ++
-      show (tarDir </> filePath ev)
+      show (tarDir </> filePath' ev)
 handleTargetEvent _ _ tarDir ev@Deleted {}
-  | not (isDirectory ev) && takeExtension (filePath ev) == shutdownExtension =
+  | not (isDirectory ev) && takeExtension (filePath' ev) == shutdownExtension =
     Logging.logInfo $ "User shutdown file deleted " ++
-      show (tarDir </> filePath ev)
+      show (tarDir </> filePath' ev)
 handleTargetEvent lock _ tarDir ev =
   handleGenericEvent lock tarDir tarDir ev
 
@@ -257,7 +267,7 @@ handleDir lock monitors curDir tarDir event =
 recapDir :: Lock -> Monitors -> FilePath -> IO ()
 recapDir lock monitors dir =
   do files <- getDirectoryContents dir
-     let files' = filter isMonitorPath files
+     let files' = map toInotifyPath $ filter isMonitorPath files
      mapM_ sendEvent files'
   where sendEvent file =
           handleTargetEvent lock monitors dir Created { isDirectory = False
@@ -281,7 +291,7 @@ watchDir lock tarDir inotify = watchDir' tarDir
                  let events = watchDirEvents dir
                  Logging.logInfo $ "Watch directory " ++ show dir
                  monitors <- newMVar Set.empty
-                 wd <- addWatch inotify events dir
+                 wd <- addWatch inotify events (toInotifyPath dir)
                        (handleDir lock monitors dir tarDir)
                  when (dir == tarDir) $ recapDir lock monitors dir
                  () <- takeMVar lock
