@@ -299,7 +299,9 @@ module Ganeti.OpParams
   , pEnabledDataCollectors
   , pDataCollectorInterval
   , pNodeSslCerts
-  , pSshKeys
+  , pSshKeyBits
+  , pSshKeyType
+  , pRenewSshKeys
   , pNodeSetup
   , pVerifyClutter
   , pLongSleep
@@ -316,7 +318,7 @@ import qualified Ganeti.Constants as C
 import Ganeti.THH
 import Ganeti.THH.Field
 import Ganeti.Utils
-import Ganeti.JSON
+import Ganeti.JSON (GenericContainer)
 import Ganeti.Types
 import qualified Ganeti.Query.Language as Qlang
 
@@ -903,12 +905,12 @@ pPowerDelay =
 pRequiredNodes :: Field
 pRequiredNodes =
   withDoc "Required list of node names" .
-  renameField "ReqNodes " $ simpleField "nodes" [t| [NonEmptyString] |]
+  renameField "ReqNodes" $ simpleField "nodes" [t| [NonEmptyString] |]
 
 pRequiredNodeUuids :: Field
 pRequiredNodeUuids =
   withDoc "Required list of node UUIDs" .
-  renameField "ReqNodeUuids " . optionalField $
+  renameField "ReqNodeUuids" . optionalField $
   simpleField "node_uuids" [t| [NonEmptyString] |]
 
 pRestrictedCommand :: Field
@@ -1226,7 +1228,7 @@ pInstOsParamsSecret :: Field
 pInstOsParamsSecret =
   withDoc "Secret OS parameters for instance" .
   optionalField $
-  simpleField "osparams_secret" [t| JSObject (Private JSValue) |]
+  simpleField "osparams_secret" [t| JSObject (Secret JSValue) |]
 
 pPrimaryNode :: Field
 pPrimaryNode =
@@ -1346,7 +1348,7 @@ pTempOsParamsSecret :: Field
 pTempOsParamsSecret =
   withDoc "Secret OS parameters for instance reinstalls" .
   optionalField $
-  simpleField "osparams_secret" [t| JSObject (Private JSValue) |]
+  simpleField "osparams_secret" [t| JSObject (Secret JSValue) |]
 
 pShutdownTimeout :: Field
 pShutdownTimeout =
@@ -1519,7 +1521,7 @@ pOsNameChange =
 pDiskIndex :: Field
 pDiskIndex =
   withDoc "Disk index for e.g. grow disk" .
-  renameField "DiskIndex " $ simpleField "disk" [t| DiskIndex |]
+  renameField "DiskIndex" $ simpleField "disk" [t| DiskIndex |]
 
 pDiskChgAmount :: Field
 pDiskChgAmount =
@@ -1740,7 +1742,7 @@ pIAllocatorOs =
 pIAllocatorInstances :: Field
 pIAllocatorInstances =
   withDoc "IAllocator instances field" .
-  renameField "IAllocatorInstances " .
+  renameField "IAllocatorInstances" .
   optionalField $
   simpleField "instances" [t| [NonEmptyString] |]
 
@@ -1895,11 +1897,21 @@ pNodeSslCerts =
   defaultField [| False |] $
   simpleField "node_certificates" [t| Bool |]
 
-pSshKeys :: Field
-pSshKeys =
+pSshKeyBits :: Field
+pSshKeyBits =
+  withDoc "The number of bits of the SSH key Ganeti uses" .
+  optionalField $ simpleField "ssh_key_bits" [t| Positive Int |]
+
+pSshKeyType :: Field
+pSshKeyType =
+  withDoc "The type of the SSH key Ganeti uses" .
+  optionalField $ simpleField "ssh_key_type" [t| SshKeyType |]
+
+pRenewSshKeys :: Field
+pRenewSshKeys =
   withDoc "Whether to renew SSH keys" .
   defaultField [| False |] $
-  simpleField "ssh_keys" [t| Bool |]
+  simpleField "renew_ssh_keys" [t| Bool |]
 
 pNodeSetup :: Field
 pNodeSetup =
@@ -1917,4 +1929,4 @@ pLongSleep :: Field
 pLongSleep =
   withDoc "Whether to allow long instance shutdowns during exports" .
   defaultField [| False |] $
-  simpleField "long_sleep" [t| Bool |]
\ No newline at end of file
+  simpleField "long_sleep" [t| Bool |]
