@@ -1,4 +1,5 @@
-{-# LANGUAGE TemplateHaskell, CPP #-}
+{-# LANGUAGE TemplateHaskell, CPP, DeriveFunctor, DeriveFoldable,
+             DeriveTraversable #-}
 
 {-| Implementation of the Ganeti Query2 language.
 
@@ -36,12 +37,14 @@ SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
 module Ganeti.Query.Language
     ( Filter(..)
+    , filterArguments
     , FilterField
     , FilterValue(..)
     , FilterRegex -- note: we don't export the constructor, must use helpers
     , mkRegex
     , stringRegex
     , compiledRegex
+    , showFilterValue
     , Fields
     , Query(..)
     , QueryResult(..)
@@ -65,12 +68,12 @@ module Ganeti.Query.Language
 import Control.Applicative
 import Control.DeepSeq
 import Data.Foldable
-import Data.Traversable (Traversable, traverse, fmapDefault, foldMapDefault)
+import Data.Traversable (Traversable)
 import Data.Ratio (numerator, denominator)
 import Text.JSON.Pretty (pp_value)
 import Text.JSON.Types
 import Text.JSON
-#ifndef NO_REGEX_PCRE
+#ifdef VERSION_regex_pcre
 import qualified Text.Regex.PCRE as PCRE
 #endif
 
@@ -91,7 +94,8 @@ $(makeJSONInstance ''ResultStatus)
 
 -- | No-op 'NFData' instance for 'ResultStatus', since it's a single
 -- constructor data-type.
-instance NFData ResultStatus
+instance NFData ResultStatus where
+  rnf x = seq x ()
 
 -- | Check that ResultStatus is success or fail with descriptive
 -- message.
@@ -131,6 +135,7 @@ $(makeJSONInstance ''QueryTypeOp)
 $(declareSADT "QueryTypeLuxi"
   [ ("QRLock",     'C.qrLock )
   , ("QRJob",      'C.qrJob )
+  , ("QRFilter",   'C.qrFilter )
   ])
 $(makeJSONInstance ''QueryTypeLuxi)
 
@@ -164,7 +169,7 @@ instance JSON ItemType where
 -- * Sub data types for query2 queries and responses.
 
 -- | Internal type of a regex expression (not exported).
-#ifndef NO_REGEX_PCRE
+#ifdef VERSION_regex_pcre
 type RegexType = PCRE.Regex
 #else
 type RegexType = ()
@@ -189,7 +194,13 @@ data Filter a
     | GEFilter       a FilterValue  -- ^ @>=@ /field/ /value/
     | RegexpFilter   a FilterRegex  -- ^ @=~@ /field/ /regexp/
     | ContainsFilter a FilterValue  -- ^ @=[]@ /list-field/ /value/
-      deriving (Show, Eq)
+      deriving (Show, Eq, Ord, Functor, Foldable, Traversable)
+
+-- | Get the \"things\" a filter talks about. This is useful, e.g.,
+-- to decide which additional fields to fetch in a query depending
+-- on live data.
+filterArguments :: Filter a -> [a]
+filterArguments = toList
 
 -- | Serialiser for the 'Filter' data type.
 showFilter :: (JSON a) => Filter a -> JSValue
@@ -268,6 +279,12 @@ readFilterArray op args
   | op == C.qlangOpNot      = readFilterArg        NotFilter args
   | op == C.qlangOpTrue     = readFilterField      TrueFilter args
   | op == C.qlangOpEqual    = readFilterFieldValue EQFilter args
+  -- Legacy filters:
+  -- - "=" is a legacy short-cut for "==".
+  -- - "!=" is a legacy short-cut for "not (... == ..)".
+  | op == C.qlangOpEqualLegacy = readFilterFieldValue EQFilter args
+  | op == C.qlangOpNotEqual    = readFilterFieldValue
+                                   (\f v -> NotFilter $ EQFilter f v) args
   | op == C.qlangOpLt       = readFilterFieldValue LTFilter args
   | op == C.qlangOpGt       = readFilterFieldValue GTFilter args
   | op == C.qlangOpLe       = readFilterFieldValue LEFilter args
@@ -280,37 +297,13 @@ instance (JSON a) => JSON (Filter a) where
   showJSON = showFilter
   readJSON = readFilter
 
--- Traversable implementation for 'Filter'.
-traverseFlt :: (Applicative f) => (a -> f b) -> Filter a -> f (Filter b)
-traverseFlt _ EmptyFilter       = pure EmptyFilter
-traverseFlt f (AndFilter flts)  = AndFilter <$> traverse (traverseFlt f) flts
-traverseFlt f (OrFilter  flts)  = OrFilter  <$> traverse (traverseFlt f) flts
-traverseFlt f (NotFilter flt)   = NotFilter <$> traverseFlt f flt
-traverseFlt f (TrueFilter a)    = TrueFilter <$> f a
-traverseFlt f (EQFilter a fval) = EQFilter <$> f a <*> pure fval
-traverseFlt f (LTFilter a fval) = LTFilter <$> f a <*> pure fval
-traverseFlt f (GTFilter a fval) = GTFilter <$> f a <*> pure fval
-traverseFlt f (LEFilter a fval) = LEFilter <$> f a <*> pure fval
-traverseFlt f (GEFilter a fval) = GEFilter <$> f a <*> pure fval
-traverseFlt f (RegexpFilter a re)     = RegexpFilter <$> f a <*> pure re
-traverseFlt f (ContainsFilter a fval) = ContainsFilter <$> f a <*> pure fval
-
-instance Traversable Filter where
-  traverse = traverseFlt
-
-instance Functor Filter where
-  fmap = fmapDefault
-
-instance Foldable Filter where
-  foldMap = foldMapDefault
-
 -- | Field name to filter on.
 type FilterField = String
 
 -- | Value to compare the field value to, for filtering purposes.
 data FilterValue = QuotedString String
                  | NumericValue Integer
-                   deriving (Show, Eq)
+                   deriving (Show, Eq, Ord)
 
 -- | Serialiser for 'FilterValue'. The Python code just sends this to
 -- JSON as-is, so we'll do the same.
@@ -347,7 +340,7 @@ data FilterRegex = FilterRegex
 -- regular expression on the initialisation of the data structure;
 -- this might fail, if the RE is not well-formed.
 mkRegex :: (Monad m) => String -> m FilterRegex
-#ifndef NO_REGEX_PCRE
+#ifdef VERSION_regex_pcre
 mkRegex str = do
   compiled <- case PCRE.getVersion of
                 Nothing -> fail $ "regex-pcre library compiled without" ++
@@ -369,6 +362,10 @@ instance Show FilterRegex where
 instance Eq FilterRegex where
   (FilterRegex re1 _) == (FilterRegex re2 _) = re1 == re2
 
+-- | 'Ord' instance: we only compare the string versions of the regexes.
+instance Ord FilterRegex where
+  (FilterRegex re1 _) `compare` (FilterRegex re2 _) = re1 `compare` re2
+
 -- | 'JSON' instance: like for show and read instances, we work only
 -- with the string component.
 instance JSON FilterRegex where
