@@ -1,4 +1,4 @@
-{-# LANGUAGE ParallelListComp, TemplateHaskell #-}
+{-# LANGUAGE ParallelListComp, TemplateHaskell, RankNTypes #-}
 
 {-| TemplateHaskell helper for Ganeti Haskell code.
 
@@ -74,11 +74,12 @@ module Ganeti.THH ( declareSADT
                   , buildParam
                   , genException
                   , excErrMsg
+                  , ssconfConstructorName
                   ) where
 
 import Control.Arrow ((&&&), second)
 import Control.Applicative
-import Control.Lens.Type (Lens')
+import Control.Lens.Type (Lens, Lens')
 import Control.Lens (lens, set, element)
 import Control.Monad
 import Control.Monad.Base () -- Needed to prevent spurious GHC linking errors.
@@ -93,18 +94,22 @@ import Data.List
 import Data.Maybe
 import qualified Data.Map as M
 import Data.Monoid
+import qualified Data.Semigroup as Sem
 import qualified Data.Set as S
+import qualified Data.Text as T
 import Language.Haskell.TH
 import Language.Haskell.TH.Syntax (lift)
 
 import qualified Text.JSON as JSON
 import Text.JSON.Pretty (pp_value)
 
-import Ganeti.JSON
+import Ganeti.JSON (readJSONWithDesc, fromObj, DictObject(..), ArrayObject(..),
+                    maybeFromObj, mkUsedKeys, showJSONtoDict, readJSONfromDict,
+                    branchOnField, addField, allUsedKeys)
 import Ganeti.PartialParams
 import Ganeti.PyValue
 import Ganeti.THH.PyType
-
+import Ganeti.THH.Compat
 
 -- * Exported types
 
@@ -169,7 +174,7 @@ data OptionalType
 -- allows to parse any additional arguments not covered by other fields. There
 -- can be at most one such special field and it's type must be
 -- @Map String JSON.JSValue@. See also 'andRestArguments'.
-data Field = Field { fieldName        :: String
+data Field = Field { fieldName        :: T.Text
                    , fieldType        :: Q Type
                      -- ^ the type of the field, @t@ for non-optional fields,
                      -- @Maybe t@ for optional ones.
@@ -180,25 +185,25 @@ data Field = Field { fieldName        :: String
                    , fieldShow        :: Maybe (Q Exp)
                      -- ^ an optional custom serialization function of type
                      -- @t -> (JSON.JSValue, [(String, JSON.JSValue)])@
-                   , fieldExtraKeys   :: [String]
+                   , fieldExtraKeys   :: [T.Text]
                      -- ^ a list of extra keys added by 'fieldShow'
                    , fieldDefault     :: Maybe (Q Exp)
                      -- ^ an optional default value of type @t@
                    , fieldSerializeDefault :: Bool
                      -- ^ whether not presented default value will be
                      -- serialized
-                   , fieldConstr      :: Maybe String
+                   , fieldConstr      :: Maybe T.Text
                    , fieldIsOptional  :: OptionalType
                      -- ^ determines if a field is optional, and if yes,
                      -- how
-                   , fieldDoc         :: String
+                   , fieldDoc         :: T.Text
                    , fieldPresentInForthcoming :: Bool
                    }
 
 -- | Generates a simple field.
 simpleField :: String -> Q Type -> Field
 simpleField fname ftype =
-  Field { fieldName        = fname
+  Field { fieldName        = T.pack fname
         , fieldType        = ftype
         , fieldRead        = Nothing
         , fieldShow        = Nothing
@@ -207,14 +212,14 @@ simpleField fname ftype =
         , fieldSerializeDefault = True
         , fieldConstr      = Nothing
         , fieldIsOptional  = NotOptional
-        , fieldDoc         = ""
+        , fieldDoc         = T.pack ""
         , fieldPresentInForthcoming = False
         }
 
 -- | Generate an AndRestArguments catch-all field.
 andRestArguments :: String -> Field
 andRestArguments fname =
-  Field { fieldName        = fname
+  Field { fieldName        = T.pack fname
         , fieldType        = [t| M.Map String JSON.JSValue |]
         , fieldRead        = Nothing
         , fieldShow        = Nothing
@@ -223,17 +228,17 @@ andRestArguments fname =
         , fieldSerializeDefault = True
         , fieldConstr      = Nothing
         , fieldIsOptional  = AndRestArguments
-        , fieldDoc         = ""
+        , fieldDoc         = T.pack ""
         , fieldPresentInForthcoming = True
         }
 
 withDoc :: String -> Field -> Field
 withDoc doc field =
-  field { fieldDoc = doc }
+  field { fieldDoc = T.pack doc }
 
 -- | Sets the renamed constructor field.
 renameField :: String -> Field -> Field
-renameField constrName field = field { fieldConstr = Just constrName }
+renameField constrName field = field { fieldConstr = Just $ T.pack constrName }
 
 -- | Sets the default value on a field (makes it optional with a
 -- default value).
@@ -277,14 +282,14 @@ customField :: Name      -- ^ The name of the read function
             -> Field     -- ^ Updated field
 customField readfn showfn extra field =
   field { fieldRead = Just (varE readfn), fieldShow = Just (varE showfn)
-        , fieldExtraKeys = extra }
+        , fieldExtraKeys = (map T.pack extra) }
 
 -- | Computes the record name for a given field, based on either the
 -- string value in the JSON serialisation or the custom named if any
 -- exists.
 fieldRecordName :: Field -> String
 fieldRecordName (Field { fieldName = name, fieldConstr = alias }) =
-  fromMaybe (camelCase name) alias
+  maybe (camelCase . T.unpack $ name) T.unpack alias
 
 -- | Computes the preferred variable name to use for the value of this
 -- field. If the field has a specific constructor name, then we use a
@@ -293,8 +298,8 @@ fieldRecordName (Field { fieldName = name, fieldConstr = alias }) =
 fieldVariable :: Field -> String
 fieldVariable f =
   case (fieldConstr f) of
-    Just name -> ensureLower name
-    _ -> map (\c -> if c == '-' then '_' else c) $ fieldName f
+    Just name -> ensureLower . T.unpack $ name
+    _ -> map (\c -> if c == '-' then '_' else c) . T.unpack . fieldName $ f
 
 -- | Compute the actual field type (taking into account possible
 -- optional status).
@@ -306,15 +311,16 @@ actualFieldType f | fieldIsOptional f `elem` [NotOptional, AndRestArguments] = t
 -- | Checks that a given field is not optional (for object types or
 -- fields which should not allow this case).
 checkNonOptDef :: (Monad m) => Field -> m ()
-checkNonOptDef (Field { fieldIsOptional = OptionalOmitNull
-                      , fieldName = name }) =
-  fail $ "Optional field " ++ name ++ " used in parameter declaration"
-checkNonOptDef (Field { fieldIsOptional = OptionalSerializeNull
-                      , fieldName = name }) =
-  fail $ "Optional field " ++ name ++ " used in parameter declaration"
-checkNonOptDef (Field { fieldDefault = (Just _), fieldName = name }) =
-  fail $ "Default field " ++ name ++ " used in parameter declaration"
-checkNonOptDef _ = return ()
+checkNonOptDef field
+  | fieldIsOptional field == OptionalOmitNull       = failWith kOpt
+  | fieldIsOptional field == OptionalSerializeNull  = failWith kOpt
+  | isJust (fieldDefault field)                     = failWith kDef
+  | otherwise                                       = return ()
+  where failWith kind = fail $ kind ++ " field " ++ name
+                                    ++ " used in parameter declaration"
+        name = T.unpack (fieldName field)
+        kOpt = "Optional"
+        kDef = "Default"
 
 -- | Construct a function that parses a field value. If the field has
 -- a custom 'fieldRead', it's applied to @o@ and used. Otherwise
@@ -324,8 +330,9 @@ parseFn :: Field   -- ^ The field definition
         -> Q Exp   -- ^ The resulting function that parses a JSON message
 parseFn field o =
   let fnType = [t| JSON.JSValue -> JSON.Result $(fieldType field) |]
-      expr = maybe [| readJSONWithDesc $(stringE $ fieldName field) |]
-                   (`appE` o) (fieldRead field)
+      expr = maybe
+               [| readJSONWithDesc $(stringE . T.unpack $ fieldName field) |]
+               (`appE` o) (fieldRead field)
   in sigE expr fnType
 
 -- | Produces the expression that will de-serialise a given
@@ -347,7 +354,7 @@ loadFnOpt :: Field   -- ^ The field definition
 loadFnOpt field@(Field { fieldDefault = Just def }) expr o
   = case fieldIsOptional field of
       NotOptional -> [| $expr >>= maybe (return $def) $(parseFn field o) |]
-      _           -> fail $ "Field " ++ fieldName field ++ ":\
+      _           -> fail $ "Field " ++ (T.unpack . fieldName $ field) ++ ":\
                             \ A field can't be optional and\
                             \ have a default value at the same time."
 loadFnOpt field expr o
@@ -417,7 +424,7 @@ appConsApp cname =
 buildConsField :: Q Type -> StrictTypeQ
 buildConsField ftype = do
   ftype' <- ftype
-  return (NotStrict, ftype')
+  return (myNotStrict, ftype')
 
 -- | Builds a constructor based on a simple definition (not field-based).
 buildSimpleCons :: Name -> SimpleObject -> Q Dec
@@ -425,7 +432,7 @@ buildSimpleCons tname cons = do
   decl_d <- mapM (\(cname, fields) -> do
                     fields' <- mapM (buildConsField . snd) fields
                     return $ NormalC (mkName cname) fields') cons
-  return $ DataD [] tname [] decl_d [''Show, ''Eq]
+  return $ gntDataD [] tname [] decl_d [''Show, ''Eq]
 
 -- | Generate the save function for a given type.
 genSaveSimpleObj :: Name                            -- ^ Object type
@@ -446,7 +453,7 @@ genSaveSimpleObj tname sname opdefs fn = do
 -- The type will have a fixed list of instances.
 strADTDecl :: Name -> [String] -> Dec
 strADTDecl name constructors =
-  DataD [] name []
+  gntDataD [] name []
           (map (flip NormalC [] . mkName) constructors)
           [''Show, ''Eq, ''Enum, ''Bounded, ''Ord]
 
@@ -482,7 +489,7 @@ genToRaw traw fname tname constructors = do
 genFromRaw :: Name -> Name -> Name -> [(String, Either String Name)] -> Q [Dec]
 genFromRaw traw fname tname constructors = do
   -- signature of form (Monad m) => String -> m $name
-  sigt <- [t| (Monad m) => $(conT traw) -> m $(conT tname) |]
+  sigt <- [t| forall m. (Monad m) => $(conT traw) -> m $(conT tname) |]
   -- clauses for a guarded pattern
   let varp = mkName "s"
       varpe = varE varp
@@ -494,9 +501,9 @@ genFromRaw traw fname tname constructors = do
                      return (g, r)) constructors
   -- the otherwise clause (fallback)
   oth_clause <- do
+    let err = "Invalid string value for type " ++ (nameBase tname) ++ ": "
     g <- normalG [| otherwise |]
-    r <- [|fail ("Invalid string value for type " ++
-                 $(litE (stringL (nameBase tname))) ++ ": " ++ show $varpe) |]
+    r <- [|fail $ $(litE (stringL err)) ++ show $varpe |]
     return (g, r)
   let fun = FunD fname [Clause [VarP varp]
                         (GuardedB (clauses++[oth_clause])) []]
@@ -592,7 +599,7 @@ makeJSONInstance name = do
   let base = nameBase name
   showJ <- genShowJSON base
   readJ <- genReadJSON base
-  return [InstanceD [] (AppT (ConT ''JSON.JSON) (ConT name)) [readJ,showJ]]
+  return [gntInstanceD [] (AppT (ConT ''JSON.JSON) (ConT name)) [readJ,showJ]]
 
 -- * Template code for opcodes
 
@@ -616,10 +623,10 @@ constructorName x                = fail $ "Unhandled constructor " ++ show x
 reifyConsNames :: Name -> Q [String]
 reifyConsNames name = do
   reify_result <- reify name
-  case reify_result of
-    TyConI (DataD _ _ _ cons _) -> mapM (liftM nameBase . constructorName) cons
-    o -> fail $ "Unhandled name passed to reifyConsNames, expected\
-                \ type constructor but got '" ++ show o ++ "'"
+  case extractDataDConstructors reify_result of
+    Just cons -> mapM (liftM nameBase . constructorName) cons
+    _ -> fail $ "Unhandled name passed to reifyConsNames, expected\
+                \ type constructor but got '" ++ show reify_result ++ "'"
 
 -- | Builds the generic constructor-to-string function.
 --
@@ -723,10 +730,10 @@ genPyDefault f = maybeApp (fieldDefault f) (fieldType f)
 
 pyField :: Field -> Q Exp
 pyField f = genPyType f >>= \t ->
-            [| OpCodeField $(stringE (fieldName f))
+            [| OpCodeField $(stringE . T.unpack . fieldName $ f)
                            t
                            $(genPyDefault f)
-                           $(stringE (fieldDoc f)) |]
+                           $(stringE . T.unpack . fieldDoc $ f) |]
 
 -- | Generates a Haskell function call to "showPyClass" with the
 -- necessary information on how to build the Python class string.
@@ -766,7 +773,7 @@ genOpCodeDictObject :: Name                -- ^ Type name to use
 genOpCodeDictObject tname savefn loadfn cons = do
   tdclauses <- genSaveOpCode cons savefn
   fdclauses <- genLoadOpCode cons loadfn
-  return [ InstanceD [] (AppT (ConT ''DictObject) (ConT tname))
+  return [ gntInstanceD [] (AppT (ConT ''DictObject) (ConT tname))
            [ FunD 'toDict tdclauses
            , FunD 'fromDictWKeys fdclauses
            ]]
@@ -787,7 +794,7 @@ genOpCode name cons = do
                     fields' <- mapM (fieldTypeInfo "op") fields
                     return $ RecC (mkName cname) fields')
             cons
-  let declD = DataD [] tname [] decl_d [''Show, ''Eq]
+  let declD = gntDataD [] tname [] decl_d [''Show, ''Eq]
   let (allfsig, allffn) = genAllOpFields "allOpFields" cons
   -- DictObject
   let luxiCons = map opcodeConsToLuxiCons cons
@@ -802,8 +809,8 @@ genOpCode name cons = do
 genOpConsFields :: OpCodeConstructor -> Clause
 genOpConsFields (cname, _, _, fields, _) =
   let op_id = deCamelCase cname
-      fvals = map (LitE . StringL) . sort . nub $
-              concatMap (\f -> fieldName f:fieldExtraKeys f) fields
+      fieldnames f = map T.unpack $ fieldName f:fieldExtraKeys f
+      fvals = map (LitE . StringL) . sort . nub $ concatMap fieldnames fields
   in Clause [LitP (StringL op_id)] (NormalB $ ListE fvals) []
 
 -- | Generates a list of all fields of an opcode constructor.
@@ -879,7 +886,7 @@ genLoadOpCode opdefs fn = do
       body = DoE [st, cst]
   -- include "OP_ID" to the list of used keys
   bodyAndOpId <- [| $(return body)
-                    <* tell (mkUsedKeys $ S.singleton opidKey) |]
+                    <* tell (mkUsedKeys . S.singleton . T.pack $ opidKey) |]
   return [Clause [VarP objname] (NormalB bodyAndOpId) []]
 
 -- * Template code for luxi
@@ -911,10 +918,10 @@ genLuxiOp name cons = do
   decl_d <- mapM (\(cname, fields) -> do
                     -- we only need the type of the field, without Q
                     fields' <- mapM actualFieldType fields
-                    let fields'' = zip (repeat NotStrict) fields'
+                    let fields'' = zip (repeat myNotStrict) fields'
                     return $ NormalC (mkName cname) fields'')
             cons
-  let declD = DataD [] (mkName name) [] decl_d [''Show, ''Eq]
+  let declD = gntDataD [] (mkName name) [] decl_d [''Show, ''Eq]
   -- generate DictObject instance
   dictObjInst <- genOpCodeDictObject tname saveLuxiConstructor
                                      loadOpConstructor cons
@@ -949,7 +956,7 @@ fieldTypeInfo :: String -> Field -> Q (Name, Strict, Type)
 fieldTypeInfo field_pfx fd = do
   t <- actualFieldType fd
   let n = mkName . (field_pfx ++) . fieldRecordName $ fd
-  return (n, NotStrict, t)
+  return (n, myNotStrict, t)
 
 -- | Build an object declaration.
 buildObject :: String -> String -> [Field] -> Q [Dec]
@@ -961,7 +968,7 @@ buildObject sname field_pfx fields = do
   let name = mkName sname
   fields_d <- mapM (fieldTypeInfo field_pfx) fields
   let decl_d = RecC name fields_d
-  let declD = DataD [] name [] [decl_d] [''Show, ''Eq]
+  let declD = gntDataD [] name [] [decl_d] [''Show, ''Eq]
   ser_decls <- buildObjectSerialisation sname fields
   return $ declD:ser_decls
 
@@ -992,8 +999,15 @@ buildAccessor fnm fpfx rnm rpfx nm pfx field = do
              , Clause [ConP fnm [VarP x]] (NormalB f_body) []
              ]]
 
--- | Build lense declartions for a field, if the type of the field
--- is the same in the forthcoming and the real variant.
+-- | Build lense declartions for a field.
+--
+-- If the type of the field is the same in
+-- the forthcoming and the real variant, the lens
+-- will be a simple lens (Lens' s a).
+--
+-- Otherwise, the type will be (Lens s s (Maybe a) a).
+-- This is because the field in forthcoming variant
+-- has type (Maybe a), but the real variant has type a.
 buildLens :: (Name, Name) -- ^ names of the forthcoming constructors
           -> (Name, Name) -- ^ names of the real constructors
           -> Name -- ^ name of the type
@@ -1004,32 +1018,45 @@ buildLens :: (Name, Name) -- ^ names of the forthcoming constructors
           -> Q [Dec]
 buildLens (fnm, fdnm) (rnm, rdnm) nm pfx ar (field, i) = do
   let optField = makeOptional field
-  if fieldIsOptional field /= fieldIsOptional optField
-     then return []
-     else do
-       let lensnm = mkName $ pfx ++ fieldRecordName  field ++ "L"
-       (accnm, _, ftype) <- fieldTypeInfo pfx field
-       vars <- replicateM ar (newName "x")
-       var <- newName "val"
-       context <- newName "val"
-       let body cn cdn = NormalB
-                           . (ConE cn `AppE`)
-                           . foldl (\e (j, x) -> AppE e . VarE
-                                                   $ if i == j then var else x)
-                             (ConE cdn)
-                          $ zip [0..] vars
-       let setterE = LamE [VarP context, VarP var] $ CaseE (VarE context)
-                        [ Match (ConP fnm [ConP fdnm . set (element i) WildP
-                                             $ map VarP vars])
-                                (body fnm fdnm) []
-                        , Match (ConP rnm [ConP rdnm . set (element i) WildP
-                                             $ map VarP vars])
-                                (body rnm rdnm) []
-                        ]
-       return [ SigD lensnm $ ConT ''Lens' `AppT` ConT nm `AppT` ftype
-              , ValD (VarP lensnm)
+      isSimple = fieldIsOptional field == fieldIsOptional optField
+      lensnm = mkName $ pfx ++ fieldRecordName  field ++ "L"
+  (accnm, _, ftype) <- fieldTypeInfo pfx field
+  vars <- replicateM ar (newName "x")
+  var <- newName "val"
+  context <- newName "val"
+  jE <- [| Just |]
+  let body eJ cn cdn = NormalB
+                      . (ConE cn `AppE`)
+                      . foldl (\e (j, x) -> AppE e $
+                                                if i == j
+                                                  then if eJ
+                                                    then AppE jE (VarE var)
+                                                    else VarE var
+                                                  else VarE x)
+                        (ConE cdn)
+                     $ zip [0..] vars
+  let setterE = LamE [VarP context, VarP var] $ CaseE (VarE context)
+                   [ Match (ConP fnm [ConP fdnm . set (element i) WildP
+                                        $ map VarP vars])
+                           (body (not isSimple) fnm fdnm) []
+                   , Match (ConP rnm [ConP rdnm . set (element i) WildP
+                                        $ map VarP vars])
+                           (body False rnm rdnm) []
+                   ]
+  let lensD = ValD (VarP lensnm)
                      (NormalB  $ VarE 'lens `AppE` VarE accnm `AppE` setterE) []
-              ]
+
+  if isSimple
+     then
+       return $ (SigD lensnm $ ConT ''Lens' `AppT` ConT nm `AppT` ftype)
+              : lensD : []
+     else
+       return $ (SigD lensnm $ ConT ''Lens `AppT`
+                              ConT nm `AppT`
+                              ConT nm `AppT`
+                              (ConT ''Maybe `AppT` ftype) `AppT`
+                              ftype)
+              : lensD : []
 
 -- | Build an object that can have a forthcoming variant.
 -- This will create 3 data types: two objects, prefixed by
@@ -1038,7 +1065,7 @@ buildLens (fnm, fdnm) (rnm, rdnm) nm pfx ar (field, i) = do
 -- be a JSON object, dispatching on the "forthcoming" key.
 buildObjectWithForthcoming ::
   String -- ^ Name of the newly defined type
-  -> String -- ^ base prefix for field names; for the real and fortcoming
+  -> String -- ^ base prefix for field names; for the real and forthcoming
             -- variant, with base prefix will be prefixed with "real"
             -- and forthcoming, respectively.
   -> [Field] -- ^ List of fields in the real version
@@ -1056,10 +1083,10 @@ buildObjectWithForthcoming sname field_pfx fields = do
                       (map makeOptional fields)
   let name = mkName sname
       real_d = NormalC (mkName real_nm)
-                 [(NotStrict, ConT (mkName real_data_nm))]
+                 [(myNotStrict, ConT (mkName real_data_nm))]
       forth_d = NormalC (mkName forth_nm)
-                  [(NotStrict, ConT (mkName forth_data_nm))]
-      declD = DataD [] name [] [real_d, forth_d] [''Show, ''Eq]
+                  [(myNotStrict, ConT (mkName forth_data_nm))]
+  let declD = gntDataD [] name [] [real_d, forth_d] [''Show, ''Eq]
 
   read_body <- [| branchOnField "forthcoming"
                   (liftM $(conE $ mkName forth_nm) . JSON.readJSON)
@@ -1075,7 +1102,7 @@ buildObjectWithForthcoming sname field_pfx fields = do
                  , Clause [ConP (mkName forth_nm) [VarP x]]
                     (NormalB show_forth_body) []
                  ]
-      instJSONdecl = InstanceD [] (AppT (ConT ''JSON.JSON) (ConT name))
+      instJSONdecl = gntInstanceD [] (AppT (ConT ''JSON.JSON) (ConT name))
                      [rdjson, shjson]
   accessors <- liftM concat . flip mapM fields
                  $ buildAccessor (mkName forth_nm) forth_pfx
@@ -1100,7 +1127,7 @@ buildObjectWithForthcoming sname field_pfx fields = do
                             ]
       fromdict = FunD 'fromDictWKeys [ Clause [VarP xs]
                                        (NormalB fromDictWKeysbody) [] ]
-      instDict = InstanceD [] (AppT (ConT ''DictObject) (ConT name))
+      instDict = gntInstanceD [] (AppT (ConT ''DictObject) (ConT name))
                  [todict, fromdict]
   instArray <- genArrayObjectInstance name
                  (simpleField "forthcoming" [t| Bool |] : fields)
@@ -1127,7 +1154,7 @@ buildObjectSerialisation sname fields = do
   (loadsig, loadfn) <- genLoadObject sname
   shjson <- objectShowJSON sname
   rdjson <- objectReadJSON sname
-  let instdecl = InstanceD [] (AppT (ConT ''JSON.JSON) (ConT name))
+  let instdecl = gntInstanceD [] (AppT (ConT ''JSON.JSON) (ConT name))
                  [rdjson, shjson]
   return $ dictdecls ++ savedecls ++ [loadsig, loadfn, instdecl]
 
@@ -1163,7 +1190,7 @@ defaultFromJSArray keys xs = do
 -- See 'defaultToJSArray' and 'defaultFromJSArray'.
 genArrayObjectInstance :: Name -> [Field] -> Q Dec
 genArrayObjectInstance name fields = do
-  let fnames = concatMap (liftA2 (:) fieldName fieldExtraKeys) fields
+  let fnames = fieldsKeys fields
   instanceD (return []) (appT (conT ''ArrayObject) (conT name))
     [ valD (varP 'toJSArray) (normalB [| defaultToJSArray $(lift fnames) |]) []
     , valD (varP 'fromJSArray) (normalB [| defaultFromJSArray fnames |]) []
@@ -1177,8 +1204,13 @@ genDictObject :: (Name -> Field -> Q Exp)  -- ^ a saving function
               -> Q [Dec]
 genDictObject save_fn load_fn sname fields = do
   let name = mkName sname
+      -- newName fails in ghc 7.10 when used on keywords
+      newName' "data" = newName "data_ghcBug10599"
+      newName' "instance" = newName "instance_ghcBug10599"
+      newName' "type" = newName "type_ghcBug10599"
+      newName' s = newName s
   -- toDict
-  fnames <- mapM (newName . fieldVariable) fields
+  fnames <- mapM (newName' . fieldVariable) fields
   let pat = conP name (map varP fnames)
       tdexp = [| concat $(listE $ zipWith save_fn fnames fields) |]
   tdclause <- clause [pat] (normalB tdexp) []
@@ -1188,7 +1220,7 @@ genDictObject save_fn load_fn sname fields = do
   -- the ArrayObject instance generated from DictObject
   arrdec <- genArrayObjectInstance name fields
   -- the final instance
-  return $ [InstanceD [] (AppT (ConT ''DictObject) (ConT name))
+  return $ [gntInstanceD [] (AppT (ConT ''DictObject) (ConT name))
              [ FunD 'toDict [tdclause]
              , FunD 'fromDictWKeys [fdclause]
              ]]
@@ -1230,7 +1262,7 @@ saveObjectField fvar field = do
                      -- from the default one.
                      _ -> formatCode fvarE
     AndRestArguments -> [| M.toList $(varE fvar) |]
-  where nameE = stringE (fieldName field)
+  where nameE = stringE (T.unpack . fieldName $ field)
         fvarE = varE fvar
 
 -- | Generates the showJSON clause for a given object name.
@@ -1255,31 +1287,34 @@ loadObjectField allFields field = do
                                    $ allFields
   -- these are used in all patterns below
   let objvar = varE objVarName
-      objfield = stringE (fieldName field)
+      objfield = stringE (T.unpack . fieldName $ field)
   case (fieldDefault field, fieldIsOptional field) of
-            -- Only non-optional fields without defaults must have a value;
-            -- we treat both optional types the same, since
-            -- 'maybeFromObj' can deal with both missing and null values
-            -- appropriately (the same)
-            (Nothing, NotOptional) ->
-                  loadFn field [| fromObj $objvar $objfield |] objvar
-            -- AndRestArguments need not to be parsed at all,
-            -- they're just extracted from the list of other fields.
-            (Nothing, AndRestArguments) ->
-                  [| return . M.fromList
-                     . filter (not . (`S.member` $(otherNames)) . fst)
-                     $ $objvar |]
-            _ ->  loadFnOpt field [| maybeFromObj $objvar $objfield |] objvar
+      -- Only non-optional fields without defaults must have a value;
+      -- we treat both optional types the same, since
+      -- 'maybeFromObj' can deal with both missing and null values
+      -- appropriately (the same)
+      (Nothing, NotOptional) ->
+            loadFn field [| fromObj $objvar $objfield |] objvar
+      -- AndRestArguments need not to be parsed at all,
+      -- they're just extracted from the list of other fields.
+      (Nothing, AndRestArguments) ->
+            [| return . M.fromList
+               . filter (not . (`S.member` $(otherNames)) . T.pack . fst)
+               $ $objvar |]
+      _ ->  loadFnOpt field [| maybeFromObj $objvar $objfield |] objvar
 
--- | Generates the set of all used JSON dictionary keys for a field
-fieldDictKeys :: Field -> Exp
-fieldDictKeys field = AppE (VarE 'S.fromList)
-  . ListE . map (LitE . StringL) $ liftA2 (:) fieldName fieldExtraKeys field
+fieldsKeys :: [Field] -> [String]
+fieldsKeys fields =
+  map T.unpack $ concatMap (liftA2 (:) fieldName fieldExtraKeys) fields
 
--- | Generates the list of all used JSON dictionary keys for a list of fields
+-- | Generates the set of all used JSON dictionary keys for a list of fields
+-- The equivalent of S.fromList (map T.pack ["f1", "f2", "f3"] )
 fieldsDictKeys :: [Field] -> Exp
 fieldsDictKeys fields =
-  AppE (VarE 'S.unions) . ListE . map fieldDictKeys $ fields
+  AppE (VarE 'S.fromList)
+  . AppE (AppE (VarE 'map) (VarE 'T.pack))
+  . ListE . map (LitE . StringL)
+  $ fieldsKeys fields
 
 -- | Generates the list of all used JSON dictionary keys for a list of fields
 fieldsDictKeysQ :: [Field] -> Q Exp
@@ -1319,7 +1354,8 @@ paramFieldNames field_pfx fd =
 paramFieldTypeInfo :: String -> Field -> VarStrictTypeQ
 paramFieldTypeInfo field_pfx fd = do
   t <- actualFieldType fd
-  return (snd $ paramFieldNames field_pfx fd, NotStrict, AppT (ConT ''Maybe) t)
+  return (snd $ paramFieldNames field_pfx fd,
+          myNotStrict, AppT (ConT ''Maybe) t)
 
 -- | Build a parameter declaration.
 --
@@ -1338,8 +1374,8 @@ buildParam sname field_pfx fields = do
   fields_p <- mapM (paramFieldTypeInfo field_pfx) fields
   let decl_f = RecC name_f fields_f
       decl_p = RecC name_p fields_p
-  let declF = DataD [] name_f [] [decl_f] [''Show, ''Eq]
-      declP = DataD [] name_p [] [decl_p] [''Show, ''Eq]
+  let declF = gntDataD [] name_f [] [decl_f] [''Show, ''Eq]
+  let declP = gntDataD [] name_p [] [decl_p] [''Show, ''Eq]
   ser_decls_f <- buildObjectSerialisation sname_f fields
   ser_decls_p <- buildPParamSerialisation sname_p fields
   fill_decls <- fillParam sname field_pfx fields
@@ -1351,7 +1387,7 @@ buildParamAllFields :: String -> [Field] -> [Dec]
 buildParamAllFields sname fields =
   let vname = mkName ("all" ++ sname ++ "ParamFields")
       sig = SigD vname (AppT ListT (ConT ''String))
-      val = ListE $ map (LitE . StringL . fieldName) fields
+      val = ListE $ map (LitE . StringL . T.unpack . fieldName) fields
   in [sig, ValD (VarP vname) (NormalB val) []]
 
 -- | Generates the serialisation for a partial parameter.
@@ -1363,7 +1399,7 @@ buildPParamSerialisation sname fields = do
   (loadsig, loadfn) <- genLoadObject sname
   shjson <- objectShowJSON sname
   rdjson <- objectReadJSON sname
-  let instdecl = InstanceD [] (AppT (ConT ''JSON.JSON) (ConT name))
+  let instdecl = gntInstanceD [] (AppT (ConT ''JSON.JSON) (ConT name))
                  [rdjson, shjson]
   return $ dictdecls ++ savedecls ++ [loadsig, loadfn, instdecl]
 
@@ -1388,7 +1424,7 @@ loadPParamField field = do
   let name = fieldName field
   -- these are used in all patterns below
   let objvar = varE objVarName
-      objfield = stringE name
+      objfield = stringE . T.unpack $ name
       loadexp = [| $(varE 'maybeFromObj) $objvar $objfield |]
   loadFnOpt field loadexp objvar
 
@@ -1431,16 +1467,21 @@ fillParam sname field_pfx fields = do
   let altExp = zipWith (\l r -> AppE (AppE (VarE '(<|>)) (VarE r)) (VarE l))
       mappendExp = appCons name_p $ altExp pbinds pbinds2
       mappendClause = Clause [pConP, pConP2] (NormalB mappendExp) []
+      mappendAlias = Clause [] (NormalB $ VarE '(Sem.<>)) []
   let monoidType = AppT (ConT ''Monoid) (ConT name_p)
+  let semigroupType = AppT (ConT ''Sem.Semigroup) (ConT name_p)
   -- the instances combined
-  return [ InstanceD [] instType
+  return [ gntInstanceD [] instType
                      [ FunD 'fillParams [fclause]
                      , FunD 'toPartial [tpclause]
                      , FunD 'toFilled [tfclause]
                      ]
-         , InstanceD [] monoidType
+         , gntInstanceD [] semigroupType
+                     [ FunD '(Sem.<>) [mappendClause]
+                     ]
+         , gntInstanceD [] monoidType
                      [ FunD 'mempty [memptyClause]
-                     , FunD 'mappend [mappendClause]
+                     , FunD 'mappend [mappendAlias]
                      ]]
 
 -- * Template code for exceptions
@@ -1540,10 +1581,14 @@ genLoadExc tname sname opdefs = do
                                         (VarE exc_name))
                           (str_matches ++ [defmatch]))) []
   -- the fail expression for the second function clause
-  fail_type <- [| fail $ "Invalid exception: expected '(string, [args])' " ++
-                  "      but got " ++ show (pp_value $(varE arg_else)) ++ "'"
-                |]
+  let err = "Invalid exception: expected '(string, [args])' " ++
+            "      but got "
+  fail_type <- [| fail $ err ++ show (pp_value $(varE arg_else)) ++ "'" |]
   -- the second function clause
   let clause2 = Clause [VarP arg_else] (NormalB fail_type) []
   sigt <- [t| JSON.JSValue -> JSON.Result $(conT tname) |]
   return $ (SigD fname sigt, FunD fname [clause1, clause2])
+
+-- | Compute the ssconf constructor name from its file name.
+ssconfConstructorName :: String -> String
+ssconfConstructorName = camelCase . ("s_s_" ++)
