@@ -40,6 +40,7 @@ module Ganeti.THH.PyRPC
   , genPyUDSRpcStubStr
   ) where
 
+import Prelude hiding ((<>))
 import Control.Monad
 import Data.Char (toLower, toUpper)
 import Data.Functor
@@ -181,7 +182,7 @@ genPyUDSRpcStub className constName = liftM (header $+$) .
                                       namesToClass className stubCode
   where
     header = text "# This file is automatically generated, do not edit!" $+$
-             text "# pylint: disable-all"
+             text "# pylint: skip-file"
     stubCode =
       abstrMethod genericInvokeName [ text "method", text "*args"] $+$
       method socketPathName [] (
