@@ -57,6 +57,7 @@ module Ganeti.WConfd.Monad
   , forceConfigStateDistribution
   , readConfigState
   , modifyConfigDataErr_
+  , modifyConfigAndReturnWithLock
   , modifyConfigWithLock
   , modifyLockWaiting
   , modifyLockWaiting_
@@ -65,6 +66,7 @@ module Ganeti.WConfd.Monad
   , modifyTempResState
   , modifyTempResStateErr
   , readTempResState
+  , DistributionTarget(..)
   ) where
 
 import Control.Applicative
@@ -79,7 +81,8 @@ import Control.Monad.State
 import Control.Monad.Trans.Control
 import Data.Functor.Identity
 import Data.IORef.Lifted
-import Data.Monoid (Any(..))
+import Data.Monoid (Any(..), Monoid(..))
+import qualified Data.Semigroup as Sem
 import qualified Data.Set as S
 import Data.Tuple (swap)
 import System.Posix.Process (getProcessID)
@@ -102,6 +105,20 @@ import Ganeti.Utils.Livelock (Livelock)
 import Ganeti.WConfd.ConfigState
 import Ganeti.WConfd.TempRes
 
+-- * Monoid of the locations to flush the configuration to
+
+-- | Data type describing where the configuration has to be distributed to.
+data DistributionTarget = Everywhere | ToGroups (S.Set String) deriving Show
+
+instance Sem.Semigroup DistributionTarget where
+  Everywhere <> _ = Everywhere
+  _ <> Everywhere = Everywhere
+  (ToGroups a) <> (ToGroups b) = ToGroups (a `S.union` b)
+
+instance Monoid DistributionTarget where
+  mempty = ToGroups S.empty
+  mappend = (Sem.<>)
+
 -- * Pure data types used in the monad
 
 -- | The state of the daemon, capturing both the configuration state and the
@@ -120,7 +137,7 @@ data DaemonHandle = DaemonHandle
   -- all static information that doesn't change during the life-time of the
   -- daemon should go here;
   -- all IDs of threads that do asynchronous work should probably also go here
-  , dhSaveConfigWorker :: AsyncWorker Any ()
+  , dhSaveConfigWorker :: AsyncWorker (Any, DistributionTarget) ()
   , dhSaveLocksWorker :: AsyncWorker () ()
   , dhSaveTempResWorker :: AsyncWorker () ()
   , dhLivelock :: Livelock
@@ -130,14 +147,17 @@ mkDaemonHandle :: FilePath
                -> ConfigState
                -> GanetiLockWaiting
                -> TempResState
-               -> (IO ConfigState -> [AsyncWorker () ()]
-                                  -> ResultG (AsyncWorker Any ()))
+               -> (IO ConfigState
+                   -> [AsyncWorker DistributionTarget ()]
+                   -> ResultG (AsyncWorker (Any, DistributionTarget) ()))
                   -- ^ A function that creates a worker that asynchronously
                   -- saves the configuration to the master file.
-               -> (IO ConfigState -> ResultG (AsyncWorker () ()))
+               -> (IO ConfigState
+                   -> ResultG (AsyncWorker DistributionTarget ()))
                   -- ^ A function that creates a worker that asynchronously
                   -- distributes the configuration to master candidates
-               -> (IO ConfigState -> ResultG (AsyncWorker () ()))
+               -> (IO ConfigState
+                   -> ResultG (AsyncWorker DistributionTarget ()))
                   -- ^ A function that creates a worker that asynchronously
                   -- distributes SSConf to nodes
                -> (IO GanetiLockWaiting -> ResultG (AsyncWorker () ()))
@@ -181,7 +201,7 @@ instance MonadBaseControl IO WConfdMonadInt where
 #if MIN_VERSION_monad_control(1,0,0)
 -- Needs Undecidable instances
   type StM WConfdMonadInt b = StM WConfdMonadIntType b
-  liftBaseWith f = WConfdMonadInt . liftBaseWith
+  liftBaseWith f = WConfdMonadInt $ liftBaseWith
                    $ \r -> f (r . getWConfdMonadInt)
   restoreM = WConfdMonadInt . restoreM
 #else
@@ -247,7 +267,8 @@ modifyConfigStateErrWithImmediate f immediateFollowup = do
       then do
         logDebug $ "Triggering config write" ++
                    " together with full synchronous distribution"
-        res <- liftBase . triggerWithResult (Any True) $ dhSaveConfigWorker dh
+        res <- liftBase . triggerWithResult (Any True, Everywhere)
+                 $ dhSaveConfigWorker dh
         immediateFollowup
         wait res
         logDebug "Config write and distribution finished"
@@ -255,7 +276,8 @@ modifyConfigStateErrWithImmediate f immediateFollowup = do
         -- trigger the config. saving worker and wait for it
         logDebug $ "Triggering config write" ++
                    " and asynchronous distribution"
-        res <- liftBase . triggerWithResult (Any False) $ dhSaveConfigWorker dh
+        res <- liftBase . triggerWithResult (Any False, Everywhere)
+                 $ dhSaveConfigWorker dh
         immediateFollowup
         wait res
         logDebug "Config writer finished with local task"
@@ -294,11 +316,11 @@ modifyConfigStateWithImmediate f =
 --
 -- We need a separate call for this operation, because 'modifyConfigState' only
 -- triggers the distribution when the configuration changes.
-forceConfigStateDistribution :: WConfdMonad ()
-forceConfigStateDistribution  = do
+forceConfigStateDistribution :: DistributionTarget -> WConfdMonad ()
+forceConfigStateDistribution target = do
   logDebug "Forcing synchronous config write together with full distribution"
   dh <- daemonHandle
-  liftBase . triggerAndWait (Any True) . dhSaveConfigWorker $ dh
+  liftBase . triggerAndWait (Any True, target) . dhSaveConfigWorker $ dh
   logDebug "Forced config write and distribution finished"
 
 -- | Atomically modifies the configuration data in the WConfdMonad
@@ -379,11 +401,11 @@ readLockAllocation = liftM LW.getAllocation readLockWaiting
 -- | Modify the configuration while temporarily acquiring
 -- the configuration lock. If the configuration lock is held by
 -- someone else, nothing is changed and Nothing is returned.
-modifyConfigWithLock
-  :: (TempResState -> ConfigState -> AtomicModifyMonad ConfigState)
+modifyConfigAndReturnWithLock
+  :: (TempResState -> ConfigState -> AtomicModifyMonad (a, ConfigState))
      -> State TempResState ()
-     -> WConfdMonad (Maybe ())
-modifyConfigWithLock f tempres = do
+     -> WConfdMonad (Maybe a)
+modifyConfigAndReturnWithLock f tempres = do
   now <- liftIO getClockTime
   dh <- lift . WConfdMonadInt $ ask
   pid <- liftIO getProcessID
@@ -395,7 +417,7 @@ modifyConfigWithLock f tempres = do
   let modCS ds@(DaemonState { dsTempRes = tr }) =
         mapMOf2
           dsConfigStateL
-          (\cs -> liftM (unpackConfigResult now cs . (,) ())  (f tr cs))
+          (\cs -> liftM (unpackConfigResult now cs)  (f tr cs))
           ds
   maybeDist <- bracket
     (atomicModifyWithLens (dhDaemonState dh) dsLockWaitingL
@@ -411,18 +433,25 @@ modifyConfigWithLock f tempres = do
         _ -> return ())
     (\(res, _) -> case res of
         Ok s | S.null s ->do
-          ((), modif, dist) <- atomicModifyIORefErrLog (dhDaemonState dh)
+          ret <- atomicModifyIORefErrLog (dhDaemonState dh)
                                  (liftM swap . modCS)
           atomicModifyWithLens (dhDaemonState dh) dsTempResL $ runState tempres
-          return $ Just (modif, dist)
+          return $ Just ret
         _ -> return Nothing)
-  flip (maybe $ return Nothing) maybeDist $ \(modified, dist) -> do
+  flip (maybe $ return Nothing) maybeDist $ \(val, modified, dist) -> do
     when modified $ do
       logDebug . (++) "Triggering config write; distribution "
         $ if dist then "synchronously" else "asynchronously"
-      liftBase . triggerAndWait (Any dist) $ dhSaveConfigWorker dh
+      liftBase . triggerAndWait (Any dist, Everywhere) $ dhSaveConfigWorker dh
       logDebug "Config write finished"
     logDebug "Triggering temporary reservations write"
     liftBase . triggerAndWait_ . dhSaveTempResWorker $ dh
     logDebug "Temporary reservations write finished"
-    return $ Just ()
+    return $ Just val
+
+modifyConfigWithLock
+  :: (TempResState -> ConfigState -> AtomicModifyMonad ConfigState)
+     -> State TempResState ()
+     -> WConfdMonad (Maybe ())
+modifyConfigWithLock f = modifyConfigAndReturnWithLock f'
+  where f' tr cs = fmap ((,) ()) (f tr cs)
