@@ -78,11 +78,14 @@ import Control.Lens.At
 import Control.Monad.Error
 import Control.Monad.State
 import Control.Monad.Trans.Maybe
+import qualified Data.ByteString as BS
+import qualified Data.ByteString.UTF8 as UTF8
 import qualified Data.Foldable as F
 import Data.Maybe
 import Data.Map (Map)
 import qualified Data.Map as M
 import Data.Monoid
+import qualified Data.Semigroup as Sem
 import qualified Data.Set as S
 import System.Random
 import qualified Text.JSON as J
@@ -108,13 +111,13 @@ import qualified Ganeti.Utils.MultiMap as MM
 
 -- ** Aliases to make types more meaningful:
 
-type NodeUUID = String
+type NodeUUID = BS.ByteString
 
-type InstanceUUID = String
+type InstanceUUID = BS.ByteString
 
-type DiskUUID = String
+type DiskUUID = BS.ByteString
 
-type NetworkUUID = String
+type NetworkUUID = BS.ByteString
 
 type DRBDMinor = Int
 
@@ -158,9 +161,12 @@ instance J.JSON IPv4Reservation where
 newtype TempRes j a = TempRes { getTempRes :: MM.MultiMap j a }
   deriving (Eq, Ord, Show)
 
+instance (Ord j, Ord a) => Sem.Semigroup (TempRes j a) where
+  (TempRes x) <> (TempRes y) = TempRes $ x <> y
+
 instance (Ord j, Ord a) => Monoid (TempRes j a) where
   mempty = TempRes mempty
-  mappend (TempRes x) (TempRes y) = TempRes $ x <> y
+  mappend = (Sem.<>)
 
 instance (J.JSON j, Ord j, J.JSON a, Ord a) => J.JSON (TempRes j a) where
   showJSON = J.showJSON . getTempRes
@@ -229,7 +235,9 @@ computeDRBDMap' cfg trs =
     -- | Adds minors of a disk within the state monad
     addMinors disk = do
       let minors = getDrbdMinorsForDisk disk
-      forM_ minors $ \(minor, node) -> nodeMinor node minor %= (uuidOf disk :)
+      forM_ minors $ \(minor, node) ->
+        nodeMinor (UTF8.fromString node) minor %=
+            (UTF8.fromString (uuidOf disk) :)
 
 -- | Compute the map of used DRBD minor/nodes.
 -- Report any duplicate entries as an error.
