@@ -165,17 +165,18 @@ prop_ListJobIDs = monadicIO $ do
       extractJobIDs = (>>= genericResult (fail . show) return)
   jobs <- pick $ resize 10 (listOf1 genJobId `suchThat` (\l -> l == nub l))
   (e, f, g) <-
-    run . withSystemTempDirectory "jqueue-test." $ \tempdir -> do
+    run . withSystemTempDirectory "jqueue-test-ListJobIDs." $ \tempdir -> do
     empty_dir <- extractJobIDs $ getJobIDs [tempdir]
     mapM_ (\jid -> writeFile (tempdir </> jobFileName jid) "") jobs
     full_dir <- extractJobIDs $ getJobIDs [tempdir]
     invalid_dir <- getJobIDs [tempdir </> "no-such-dir"]
     return (empty_dir, sortJobIDs full_dir, invalid_dir)
-  stop $ conjoin [ counterexample "empty directory" $ e ==? []
-                 , counterexample "directory with valid names" $
-                   f ==? sortJobIDs jobs
-                 , counterexample "invalid directory" $ isBad g
-                 ]
+  _ <- stop $ conjoin [ counterexample "empty directory" $ e ==? []
+                      , counterexample "directory with valid names" $
+                        f ==? sortJobIDs jobs
+                      , counterexample "invalid directory" $ isBad g
+                      ]
+  return ()
 
 -- | Tests loading jobs from disk.
 prop_LoadJobs :: Property
@@ -186,7 +187,7 @@ prop_LoadJobs = monadicIO $ do
       job_s = encode job
   -- check that jobs in the right directories are parsed correctly
   (missing, current, archived, missing_current, broken) <-
-    run  . withSystemTempDirectory "jqueue-test." $ \tempdir -> do
+    run  . withSystemTempDirectory "jqueue-test-LoadJobs." $ \tempdir -> do
     let load a = loadJobFromDisk tempdir a jid
         live_path = liveJobFile tempdir jid
         arch_path = archivedJobFile tempdir jid
@@ -207,12 +208,13 @@ prop_LoadJobs = monadicIO $ do
     writeFile live_path "invalid job"
     broken <- load True
     return (missing, current, archived, missing_current, broken)
-  stop $ conjoin [ missing ==? noSuchJob
-                 , current ==? Ganeti.BasicTypes.Ok (job, False)
-                 , archived ==? Ganeti.BasicTypes.Ok (job, True)
-                 , missing_current ==? noSuchJob
-                 , counterexample "broken job" (isBad broken)
-                 ]
+  _ <- stop $ conjoin [ missing ==? noSuchJob
+                      , current ==? Ganeti.BasicTypes.Ok (job, False)
+                      , archived ==? Ganeti.BasicTypes.Ok (job, True)
+                      , missing_current ==? noSuchJob
+                      , counterexample "broken job" (isBad broken)
+                      ]
+  return ()
 
 -- | Tests computing job directories. Creates random directories,
 -- files and stale symlinks in a directory, and checks that we return
@@ -225,7 +227,7 @@ prop_DetermineDirs = monadicIO $ do
   let (valid, invalid) = splitAt (count `div` 2) $
                          map (\(QuickCheck.Positive i) -> show i) nums
   (tempdir, non_arch, with_arch, invalid_root) <-
-    run  . withSystemTempDirectory "jqueue-test." $ \tempdir -> do
+    run  . withSystemTempDirectory "jqueue-test-DetermineDirs." $ \tempdir -> do
     let arch_dir = tempdir </> jobQueueArchiveSubDir
     createDirectory arch_dir
     mapM_ (createDirectory . (arch_dir </>)) valid
@@ -237,10 +239,12 @@ prop_DetermineDirs = monadicIO $ do
     invalid_root <- determineJobDirectories (tempdir </> "no-such-subdir") True
     return (tempdir, non_arch, with_arch, invalid_root)
   let arch_dir = tempdir </> jobQueueArchiveSubDir
-  stop $ conjoin [ non_arch ==? [tempdir]
-                 , sort with_arch ==? sort (tempdir:map (arch_dir </>) valid)
-                 , invalid_root ==? [tempdir </> "no-such-subdir"]
-                 ]
+  _ <- stop $ conjoin [ non_arch ==? [tempdir]
+                      , sort with_arch ==?
+                          sort (tempdir:map (arch_dir </>) valid)
+                      , invalid_root ==? [tempdir </> "no-such-subdir"]
+                      ]
+  return ()
 
 -- | Tests the JSON serialisation for 'InputOpCode'.
 prop_InputOpCode :: MetaOpCode -> Int -> Property
