@@ -71,13 +71,6 @@ genStorageUnits = do
   num_storage_units <- choose (0, 5)
   vectorOf num_storage_units genStorageUnit
 
-genStorageUnitMap :: Gen (Map.Map String [StorageUnit])
-genStorageUnitMap = do
-  num_nodes <- choose (0,5)
-  node_uuids <- vectorOf num_nodes genName
-  storage_units_list <- vectorOf num_nodes genStorageUnits
-  return $ Map.fromList (zip node_uuids storage_units_list)
-
 -- FIXME: Generate more interesting hvparams
 -- | Generate Hvparams
 genHvParams :: Gen Objects.HvParams
@@ -99,7 +92,7 @@ instance Arbitrary Rpc.RpcCallInstanceList where
   arbitrary = Rpc.RpcCallInstanceList <$> arbitrary
 
 instance Arbitrary Rpc.RpcCallNodeInfo where
-  arbitrary = Rpc.RpcCallNodeInfo <$> genStorageUnitMap <*> genHvSpecs
+  arbitrary = Rpc.RpcCallNodeInfo <$> genStorageUnits <*> genHvSpecs
 
 -- | Generates per-instance console info params for the 'InstanceConsoleInfo'
 -- call.
@@ -122,7 +115,8 @@ runOfflineTest :: (Rpc.Rpc a b, Eq b, Show b) => a -> Property
 runOfflineTest call =
   forAll (arbitrary `suchThat` Objects.nodeOffline) $ \node -> monadicIO $ do
       res <- run $ Rpc.executeRpcCall [node] call
-      stop $ res ==? [(node, Left Rpc.OfflineNodeError)]
+      _ <- stop $ res ==? [(node, Left Rpc.OfflineNodeError)]
+      return ()
 
 prop_noffl_request_allinstinfo :: Rpc.RpcCallAllInstancesInfo -> Property
 prop_noffl_request_allinstinfo = runOfflineTest
